#$Id$
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2016 Veritas Technologies LLC.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# VERITAS TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
#
# The  Licensed  Software  and  Documentation  are  deemed  to be commercial
# computer  software  as  defined  in  FAR  12.212 and subject to restricted
# rights  as  defined in FAR Section 52.227-19 "Commercial Computer Software
# - Restricted  Rights"  and  DFARS 227.7202,  et seq.  "Commercial Computer
# Software  and  Commercial Computer Software Documentation," as applicable,
# and any successor regulations, whether delivered by Veritas as on premises
# or  hosted  services.  Any  use,  modification,  reproduction  release,
# performance,  display  or  disclosure  of  the  Licensed  Software  and
# Documentation by the U.S. Government shall be solely in accordance with
# the terms of this Agreement. $

package VRTS::Enclosure::Capacities;
use strict;
use Carp;
require Exporter;
use File::Spec;
use Data::Dumper;

require VRTS::Constants::P_ARRAY_CAPACITY;
require VRTS::Constants::P_ARRAY_CAPABILITY;

use vars qw($VERSION @ISA @EXPORT);

require JSON;

my %method_map = ("HICOMMAND" => "HiCommand", "NAVISPHERE" => "NaviSphere", "NETAPP" => "NetApp");

# This way we know that this is only for a static set of array/method types.
my %capacities_hash = ("physical_size" => 0,
                  "allocated_size" => 0,
                  "spare_size" => 0,
                  "unallocated_size" => 0,
                  "rg_total_size" => 0,
                  "rg_addressable_size" => 0,
                  "rg_overhead_size" => 0,
                  "rg_free_size" => 0,
                  "logical_total_size" => 0,
                  "logical_configured_size" => 0,
                  "logical_assigned_size" => 0,
                  "logical_unassigned_size" => 0,
                  "logical_admin_size" => 0,
                  "logical_reptarget_size" => 0,
                  "subs_logical_total_size" => 0,
                  "subs_logical_configured_size" => 0,
                  "subs_logical_assigned_size" => 0,
                  "subs_logical_unassigned_size" => 0,
                  "subs_logical_admin_size" => 0,
                  "ext_physical" => 0,
                  "subs_logical_reptarget_size" => 0);

@ISA = qw(Exporter);
@EXPORT = qw(compute_capacities report_capabilities);
             
$VERSION = '5.0';

my ($array_tb, $pdev_tb, $ldev_tb, $ldev_map_tb, $vendor_cap, $vendor_handle, $dg_tb, $rg_tb);
my $method;

sub report_capabilities
{
    my $agent = shift;

    $method = $agent->{NAME};
    my $context = $agent->get_context();
    my $e_vdid = $context->{"enclosure_vdid"};
    my $compute = $context->{"compute_capacities"};

    my $array_cap_tb = $agent->gettable("P_ARRAY_CAPACITY");

    return if (not defined $e_vdid or $e_vdid eq "");

    if (defined $compute and $compute == 1)
    {
        VRTS::Logger::log('debug', "agentlet does want generic mechanism to capability reporting");
    }
    else
    {
        VRTS::Logger::log('debug', "agentlet does not want generic mechanism for capability reporting");
        return;
    }

    # One more check to see if the table(s) are marked as ignored, in that case, 
    #  the capacities need not be re-computed.
    if (defined $array_cap_tb->{IGNORE} and $array_cap_tb->{IGNORE} == 1)
    {
        VRTS::Logger::log('debug', "Capability computations need to be ignored as part of this run, skipping");
        return;
    }

    my $mod_method = $method;
    $mod_method = $method_map{$mod_method} 
          if (defined $method_map{$mod_method} and $method_map{$mod_method} ne "");

    my $module_name = "VRTS::Enclosure::Discover::".$mod_method;
    if (eval "require $module_name")
    {
        $vendor_handle = &{\&{"${module_name}::initialize"}};
    }

    return if ($@);

    $vendor_cap = $vendor_handle->provide_capabilities();
    my $all_capabilities = VRTS::Enclosure::Utils::get_capabilities_hash();
    my $cab_tb = $agent->gettable("P_ARRAY_CAPABILITY");

    while(my($key, $value) = each (%{$all_capabilities}))
    {
        my @cab_row;

        $cab_row[&VRTS::Constants::P_ARRAY_CAPABILITY::ARRAY_ID] = $e_vdid;
        $cab_row[&VRTS::Constants::P_ARRAY_CAPABILITY::ARRAY_PROP_ID] = $value->{index};
        $cab_row[&VRTS::Constants::P_ARRAY_CAPABILITY::VALUE] = 0;

        $cab_tb->addrow(\@cab_row);
    }

    while(my($key, $value) = each (%{$vendor_cap}))
    {
        my @cab_row;

        $cab_row[&VRTS::Constants::P_ARRAY_CAPABILITY::ARRAY_ID] = $e_vdid;
        $cab_row[&VRTS::Constants::P_ARRAY_CAPABILITY::ARRAY_PROP_ID] = $all_capabilities->{$key}->{index};
        $cab_row[&VRTS::Constants::P_ARRAY_CAPABILITY::VALUE] = $value;

        $cab_tb->addrow(\@cab_row);
    }

    # Last gasp attempt to see if the agentlet as part of discovery, wants to force some capabilities.
    my $array_caps = $context->{"enforce_capability"};
    if (defined $array_caps)
    {
        while(my($key, $value) = each (%{$array_caps}))
        {
            my @cab_row;

            $cab_row[&VRTS::Constants::P_ARRAY_CAPABILITY::ARRAY_ID] = $e_vdid;
            $cab_row[&VRTS::Constants::P_ARRAY_CAPABILITY::ARRAY_PROP_ID] = $all_capabilities->{$key}->{index};
            $cab_row[&VRTS::Constants::P_ARRAY_CAPABILITY::VALUE] = $value;

            $cab_tb->addrow(\@cab_row);
        }
    }

    #$cab_tb->print_table();

    return 0;
}

sub compute_capacities
{
    my $agent = shift;

    $method = $agent->{NAME};
    my $context = $agent->get_context();
    my $e_vdid = $context->{"enclosure_vdid"};
    my $compute = $context->{"compute_capacities"};

    my $array_cap_tb = $agent->gettable("P_ARRAY_CAPACITY");

    return if (not defined $e_vdid or $e_vdid eq "");

    VRTS::Logger::log('debug', "Trying to compute capacities at array object for method [$method] for enclosure [$e_vdid]");

    if (defined $compute and $compute == 1)
    {
        VRTS::Logger::log('debug', "agentlet does want generic formula to be used for capacities to be calculated for [$method]");
    }
    else
    {
        VRTS::Logger::log('debug', "agentlet does not want generic formula used for capacities to be calculated for [$method], skipping");
        return;
    }

    # One more check to see if the table(s) are marked as ignored, in that case, the capacities need not be re-computed.
    if (defined $array_cap_tb->{IGNORE} and $array_cap_tb->{IGNORE} == 1)
    {
        VRTS::Logger::log('debug', "Capacity computations need to be ignored as part of this run, skipping");
        return;
    }

    my $mod_method = $method;
    $mod_method = $method_map{$mod_method} if (defined $method_map{$mod_method} and $method_map{$mod_method} ne "");
    
    my $module_name = "VRTS::Enclosure::Discover::".$mod_method;
    if (eval "require $module_name")
    {
        $vendor_handle = &{\&{"${module_name}::initialize"}};
    }

    return if ($@);

    $vendor_cap = $vendor_handle->provide_capabilities();

    my $array_caps = $context->{"enforce_capability"};
    if (defined $array_caps)
    {
        while(my($key, $value) = each (%{$array_caps}))
        {
            $vendor_cap->{$key} = $value; 
        }
    }

    # Now that is from a list of methods which are of our interest, then we need to ensure
    #  that various capacities need to be included and reported on p_array_subsystem table.

    $array_tb = $agent->gettable("P_ARRAY_SUBSYSTEM");
    $pdev_tb = $agent->gettable("P_ARRAY_PDEV");
    $ldev_tb = $agent->gettable("P_ARRAY_LDEV");
    $ldev_map_tb = $agent->gettable("P_ARRAY_LDEV_MAP");
    $dg_tb = $agent->gettable("P_ARRAY_DEVICEGROUP_MAP");
    $rg_tb = $agent->gettable("P_ARRAY_DEVICE_GROUP");

    my $array_row = $array_tb->getrowwithvalue(&VRTS::Constants::P_ARRAY_SUBSYSTEM::ID, $e_vdid);
    return if (not defined $array_row);

    _compute_physical_capacities($agent, $method, $array_row);

    _compute_rg_capacities($agent, $method, $array_row);

    _compute_logical_capacities($agent, $method, $array_row);

    # Defaulting all capacities to zero first for starters.
    my $all_capacities_hash = VRTS::Enclosure::Utils::get_capacities_hash();
    while(my($key, $value) = each (%{$all_capacities_hash}))
    {
        my @cap_row;

        $cap_row[&VRTS::Constants::P_ARRAY_CAPACITY::ARRAY_ID] = $e_vdid;
        $cap_row[&VRTS::Constants::P_ARRAY_CAPACITY::ARRAY_PROP_ID] = $value->{index};
        $cap_row[&VRTS::Constants::P_ARRAY_CAPACITY::VALUE] = 0;

        $array_cap_tb->addrow(\@cap_row);
    }

    # Now it is time to report the capacities into the 
    while(my($key, $value) = each (%capacities_hash))
    {
        my @cap_row;

        $cap_row[&VRTS::Constants::P_ARRAY_CAPACITY::ARRAY_ID] = $e_vdid;
        $cap_row[&VRTS::Constants::P_ARRAY_CAPACITY::ARRAY_PROP_ID] = $all_capacities_hash->{$key}->{index};
        $cap_row[&VRTS::Constants::P_ARRAY_CAPACITY::VALUE] = $value;

        $array_cap_tb->addrow(\@cap_row);
    }

    # Checking if the discovery has sent some pre-computed capacities
    #   if so, then use those to override the computed values.

    if (defined $context->{overwrite_capacities})
    {
        while ((my($key, $value) = each (%{$context->{overwrite_capacities}})))
        {
            my @cap_row;

            $cap_row[&VRTS::Constants::P_ARRAY_CAPACITY::ARRAY_ID] = $e_vdid;
            $cap_row[&VRTS::Constants::P_ARRAY_CAPACITY::ARRAY_PROP_ID] = $key;
            $cap_row[&VRTS::Constants::P_ARRAY_CAPACITY::VALUE] = $value;

            $array_cap_tb->addrow(\@cap_row);
        }
    }

    #$array_cap_tb->print_table();
    #_print_caps_in_gb();
 
    return 0;
}


sub _print_caps_in_gb
{
    my %gb_capacities_hash;

    while (my($key,$value) = each(%capacities_hash))
    {
        $gb_capacities_hash{$key} = $value;
        if ($value != 0)
        {
            $gb_capacities_hash{$key} = ($value / 1024 / 1024 / 1024);
        }
    }

    return 0;
}

sub _compute_rg_capacities
{
    my ($agent, $method, $array_row) = @_;

    return if (not defined $vendor_cap or $vendor_cap->{SHOW_RG} == 0);

    my $rg_rows = $rg_tb->getallrowswithvalue(&VRTS::Constants::P_ARRAY_DEVICE_GROUP::TYPE, 'RG');
    next if (not defined $rg_rows);

    foreach my $rg_r (@{$rg_rows})
    {
        next if ($rg_r->[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::RAID_LEVEL] eq 'dummy');

        $capacities_hash{rg_total_size} += $rg_r->[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::RAW_SIZE];
        $capacities_hash{rg_overhead_size} += $rg_r->[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::OVERHEAD_SIZE];
        $capacities_hash{rg_addressable_size} += $rg_r->[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::DEVICE_SIZE];
        $capacities_hash{rg_free_size} += $rg_r->[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::FREE_SIZE];
    }

    if ($method eq 'HICOMMAND') {
        $capacities_hash{rg_overhead_size} = $capacities_hash{rg_total_size} -
                                                ($capacities_hash{rg_addressable_size} + $capacities_hash{rg_free_size});
    }
    if ($method eq 'NAVISPHERE' or $method eq '3PAR')
    {
        my $rg_rows = $rg_tb->getallrowswithvalue(&VRTS::Constants::P_ARRAY_DEVICE_GROUP::TYPE, 'TP');
        next if (not defined $rg_rows);

        foreach my $rg_r (@{$rg_rows})
        {
            $capacities_hash{rg_total_size} += $rg_r->[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::RAW_SIZE];
            $capacities_hash{rg_overhead_size} += ($rg_r->[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::RAW_SIZE] -
                                                           $rg_r->[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::TOTAL_SIZE]);
            $capacities_hash{rg_free_size} += $rg_r->[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::FREE_SIZE];
            if ($method eq '3PAR') {
                $capacities_hash{rg_addressable_size} += ($rg_r->[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::TOTAL_SIZE] -
                                                            $rg_r->[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::FREE_SIZE]);
            }          
        }
    } 
    elsif ($method eq 'XIV') 
    {
        my $rg_rows = $rg_tb->getallrowswithvalue(&VRTS::Constants::P_ARRAY_DEVICE_GROUP::TYPE, 'TP');
        return if (not defined $rg_rows);

        foreach my $rg_r (@{$rg_rows}) 
        {
            # Add thin pool hard_size_MiB to rg_total_size
            $capacities_hash{rg_total_size} += 
                                $rg_r->[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::TOTAL_SIZE] * 2;

            # Add thin pool hard_size_MiB to the overhead
            $capacities_hash{rg_overhead_size} += $rg_r->[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::TOTAL_SIZE];

            # Add the hard_size_MiB to rg_free_size for now
            # Need to subtract consumed size from rg_free_size in _compute_logical_capacities() in order to get true free size
            $capacities_hash{rg_free_size} += 
                                $rg_r->[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::TOTAL_SIZE];
        }
    }
    elsif ($method =~ /NetApp/i)
    {
        #For NetApp case, we need to also account trad Vols in this case into the aggr capacity as well.
        my $vol_rows = $rg_tb->getallrowswithvalue(&VRTS::Constants::P_ARRAY_DEVICE_GROUP::TYPE, 'tradVol');

        foreach my $rg_r (@{$vol_rows})
        {
            next if ($rg_r->[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::RAID_LEVEL] eq 'dummy');

            $capacities_hash{rg_total_size} += $rg_r->[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::RAW_SIZE];
            $capacities_hash{rg_overhead_size} += $rg_r->[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::OVERHEAD_SIZE];
            $capacities_hash{rg_addressable_size} += $rg_r->[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::DEVICE_SIZE];
            $capacities_hash{rg_free_size} += $rg_r->[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::FREE_SIZE];
        }
    }

    return 0;
}

sub _compute_physical_capacities
{
    my ($agent, $method, $array_row) = @_;
    my $fn = "_compute_physical_capacities";

    $capacities_hash{allocated_size} = 0;
    $capacities_hash{physical_size} = 0;
    $capacities_hash{available_size} = 0;
    $capacities_hash{spare_size} = 0;
    $capacities_hash{unallocated_size} = 0;

    my $pdev_rows = $pdev_tb->each_current();
    foreach my $pdev_elem (@{$pdev_rows})
    {
        if ($pdev_elem->[1]->[&VRTS::Constants::P_ARRAY_PDEV::STATUS] == 0)
        {
            $capacities_hash{physical_size} += 
                $pdev_elem->[1]->[&VRTS::Constants::P_ARRAY_PDEV::SIZE];

            $capacities_hash{spare_size} += 
                $pdev_elem->[1]->[&VRTS::Constants::P_ARRAY_PDEV::SPARE_SIZE];

            $capacities_hash{available_size} += $pdev_elem->[1]->[&VRTS::Constants::P_ARRAY_PDEV::AVAILABLE];

            if (defined $vendor_cap and $vendor_cap->{SHOW_RG} == 1 and $method ne 'XIV')
            {
                # For every pdev, which is part of atleast one RG (raid|array group), account it into allocated size.
                my @tmp_cols = (&VRTS::Constants::P_ARRAY_DEVICEGROUP_MAP::DEVICE_ID, 
                                    &VRTS::Constants::P_ARRAY_DEVICEGROUP_MAP::DEVICE_TYPE);
                my @tmp_vals = ($pdev_elem->[1]->[&VRTS::Constants::P_ARRAY_PDEV::ID], 'PDEV');

                my $dg_rows = $dg_tb->getallrowswithvaluez(\@tmp_cols, \@tmp_vals);
         
                if (defined $dg_rows and scalar(@{$dg_rows}) >= 1)
                {
                    # In clariion case, a spare disk can be part of RG/TP
                    # dont consider spare as allocated
                    next if ($method eq 'NAVISPHERE' and $pdev_elem->[1]->[&VRTS::Constants::P_ARRAY_PDEV::SPARE_SIZE] != 0);
                        
                    if ($pdev_elem->[1]->[&VRTS::Constants::P_ARRAY_PDEV::SPARE_SIZE] != 0 and
                             ($pdev_elem->[1]->[&VRTS::Constants::P_ARRAY_PDEV::SIZE] !=
                                   $pdev_elem->[1]->[&VRTS::Constants::P_ARRAY_PDEV::SPARE_SIZE]))
                    {
                        $capacities_hash{allocated_size} +=
                             ($pdev_elem->[1]->[&VRTS::Constants::P_ARRAY_PDEV::SIZE] - $pdev_elem->[1]->[&VRTS::Constants::P_ARRAY_PDEV::SPARE_SIZE]);
                    }
                    else
                    {
                        $capacities_hash{allocated_size} += $pdev_elem->[1]->[&VRTS::Constants::P_ARRAY_PDEV::SIZE];
                    }
                }
            }                
        }
    }

    if ( $capacities_hash{physical_size} == 0)
    {
        VRTS::Logger::log('debug', "$fn: Total physical size discovered as 0");
        return;
    }

    # Set the total storage on the enclosure
    $array_row->[&VRTS::Constants::P_ARRAY_SUBSYSTEM::PHYSICAL_SIZE] = $capacities_hash{physical_size};
    $array_tb->addrow($array_row);

    if ((defined $vendor_cap and $vendor_cap->{SHOW_RG} == 0) or $method eq '3PAR')
    {
        if (defined $array_row->[&VRTS::Constants::P_ARRAY_SUBSYSTEM::ALLOCATED_SIZE] and
                $array_row->[&VRTS::Constants::P_ARRAY_SUBSYSTEM::ALLOCATED_SIZE] ne 'COLUMNDEFAULTVALUE' and 
                $array_row->[&VRTS::Constants::P_ARRAY_SUBSYSTEM::ALLOCATED_SIZE] != 0)
        {
            if ($method ne 'SYMCLI')
            {
                $capacities_hash{allocated_size} = $array_row->[&VRTS::Constants::P_ARRAY_SUBSYSTEM::ALLOCATED_SIZE];
            }
            #3PAR includes 'Spare' capacity also in 'Allocated' capacity.
            #We should remove this from our 'allocated' as we show 'spare'
            #capacity differently.
            if ($method eq '3PAR') {
                $capacities_hash{allocated_size} -= $capacities_hash{spare_size};
            }
        }
   

        if (defined $array_row->[&VRTS::Constants::P_ARRAY_SUBSYSTEM::PHYSICAL_SIZE] and
               $array_row->[&VRTS::Constants::P_ARRAY_SUBSYSTEM::PHYSICAL_SIZE] ne 'COLUMNDEFAULTVALUE' and
               $array_row->[&VRTS::Constants::P_ARRAY_SUBSYSTEM::PHYSICAL_SIZE] != 0)
        {
            if ($method ne 'SYMCLI')
            {
                $capacities_hash{physical_size} = $array_row->[&VRTS::Constants::P_ARRAY_SUBSYSTEM::PHYSICAL_SIZE];
            }
        }
    }
    elsif ($method eq 'XIV')
    {
        $capacities_hash{allocated_size} = 
                  ($capacities_hash{physical_size} - $capacities_hash{spare_size});
    }

    if ($capacities_hash{allocated_size} != 0)
    {
        $capacities_hash{unallocated_size} = 
             ($capacities_hash{physical_size} - ($capacities_hash{allocated_size} + $capacities_hash{spare_size}));
    }
    else
    {
        if ($capacities_hash{available_size} > 0)
        {
            $capacities_hash{unallocated_size} = $capacities_hash{available_size};

            $capacities_hash{allocated_size} = ($capacities_hash{physical_size} - $capacities_hash{spare_size} -
                                               $capacities_hash{unallocated_size});
        }
        else
        {
            $capacities_hash{unallocated_size} = $capacities_hash{physical_size} - $capacities_hash{spare_size};
        }
    }

}

sub _compute_logical_capacities
{
    my ($agent, $method, $array_row) = @_;

    return if (defined $vendor_cap and $vendor_cap->{SHOW_LDEV} == 0);

    my $logical_remote_rep = 0;
    my $subs_logical_remote_rep = 0;
   
    my $ldev_rows = $ldev_tb->each_current();
    my $ignore_logical_caps;
    foreach my $ldev_elem (@{$ldev_rows})
    {
        $ignore_logical_caps = 0; # default dont ignore any logical capacity computations.

        if ($ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::CONSUMED_LUN_SIZE] ne 'COLUMNDEFAULTVALUE' and
                       $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::CONSUMED_LUN_SIZE] > 0) 
        {
            # If it is a thick lun, we need to consider the lun size 
            if ($method eq 'NAVISPHERE')
            { 
                if ($ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::TYPE] eq 'THICKLUN') 
                {
                    $capacities_hash{rg_addressable_size} += 
	                            $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::SIZE];
                    $capacities_hash{logical_total_size} +=
                                    $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::SIZE];
                    $capacities_hash{rg_overhead_size} += ($ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::CONSUMED_RAW_CAPACITY] - 
                                                        $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::SIZE]);
                }
                else
                {
                    $capacities_hash{rg_addressable_size} += 
		     		$ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::CONSUMED_LUN_SIZE];
                    $capacities_hash{logical_total_size} +=
                                $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::CONSUMED_LUN_SIZE];
                    $capacities_hash{rg_overhead_size} += ($ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::CONSUMED_RAW_CAPACITY] - 
                                                        $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::CONSUMED_LUN_SIZE]);
                }
            }
            else
            {
                if ($method eq 'XIV') {
                    # Since we are here, it means that for the TP to which this thin lun belongs,
                    # we have added the hard_size_MB value to rg_free_size.
                    # Adjust consumed size from rg_free_size to get the actual free size 
                    $capacities_hash{rg_free_size} -= 
                                $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::CONSUMED_LUN_SIZE];
                }
                $capacities_hash{logical_total_size} +=
                      $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::CONSUMED_LUN_SIZE];
            }
        }
        else
        {
            $capacities_hash{logical_total_size} += $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::SIZE];
        }

        $capacities_hash{subs_logical_total_size} += $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::SIZE];


        if (defined $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::REPLICATION] and 
             (($ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::REPLICATION] == 1) or
              ($ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::REPLICATION] == 4)))
        {
            my $ldev_row = $ldev_map_tb->getrowwithvalue(&VRTS::Constants::P_ARRAY_LDEV_MAP::DEST_ID,
                                $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::ID]);
            if (defined $ldev_row)
            {
                if (($ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::CONSUMED_LUN_SIZE] ne 'COLUMNDEFAULTVALUE' and
                        $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::CONSUMED_LUN_SIZE] > 0) or
                            $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::IS_ONDEMAND] == 1)
                {
                    $capacities_hash{logical_reptarget_size} += 
                            $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::CONSUMED_LUN_SIZE]; 
                }
                else
                {
                    $capacities_hash{logical_reptarget_size} += $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::SIZE]; 
                }

                $capacities_hash{subs_logical_reptarget_size} += $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::SIZE];

                $ignore_logical_caps = 1;
            }
        }

        if (defined $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::REPLICATION] and
             (($ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::REPLICATION] == 3) or
              ($ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::REPLICATION] == 5)))
        {
            if (($ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::CONSUMED_LUN_SIZE] ne 'COLUMNDEFAULTVALUE' and
                       $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::CONSUMED_LUN_SIZE] > 0) or
                            $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::IS_ONDEMAND] == 1)
            {
                $capacities_hash{logical_reptarget_size} += 
                        $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::CONSUMED_LUN_SIZE]; 
            }
            else
            {
                $capacities_hash{logical_reptarget_size} += $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::SIZE]; 
            }

            $capacities_hash{subs_logical_reptarget_size} += $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::SIZE];

            $ignore_logical_caps = 1;
        }

        if ($method eq 'HICOMMAND' or $method eq 'NAVISPHERE' or $method eq 'SYMCLI') 
        {
            next if ($ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::TYPE] eq 'SEGMENT');
        }

        if (defined $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::TYPE] and
             ($ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::TYPE] eq 'CommandDevice' or
              $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::TYPE] eq 'Meta' or
              $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::TYPE] eq 'META' or
              $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::TYPE] eq 'ADMINLUN'))
        {

            if ($ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::CONSUMED_LUN_SIZE] ne 'COLUMNDEFAULTVALUE' and
                       $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::CONSUMED_LUN_SIZE] > 0)
            {
                $capacities_hash{logical_admin_size} += $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::CONSUMED_LUN_SIZE];
            }
            else
            {
                $capacities_hash{logical_admin_size} += $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::SIZE];
            }

            $capacities_hash{subs_logical_admin_size} += $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::SIZE];
            $ignore_logical_caps = 1;
        }

        next if ($ignore_logical_caps == 1);

        if (defined $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::STATUS] and
                $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::STATUS] == 1)
        {
            if ($ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::TYPE] ne 'THICKLUN' and 
                   $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::CONSUMED_LUN_SIZE] ne 'COLUMNDEFAULTVALUE' and
                       $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::CONSUMED_LUN_SIZE] >= 0)
            {
                $capacities_hash{logical_assigned_size} += $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::CONSUMED_LUN_SIZE];
            }
            else
            {
                $capacities_hash{logical_assigned_size} += $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::SIZE];
            }

            $capacities_hash{subs_logical_assigned_size} += $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::SIZE];
        }
        elsif (defined $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::STATUS] and
                $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::STATUS] == 0 and 
                $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::TYPE] ne 'SOURCELUN')
        {
            if ($ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::TYPE] ne 'THICKLUN' and
                   $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::CONSUMED_LUN_SIZE] ne 'COLUMNDEFAULTVALUE' and
                       $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::CONSUMED_LUN_SIZE] >= 0)
            {
                $capacities_hash{logical_unassigned_size} += 
                           $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::CONSUMED_LUN_SIZE];
            }
            else
            {
                $capacities_hash{logical_unassigned_size} += 
                              $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::SIZE];
            }

            $capacities_hash{subs_logical_unassigned_size} += $ldev_elem->[1]->[&VRTS::Constants::P_ARRAY_LDEV::SIZE];
        }
    }

    if ($method =~ /NetApp/i)
    {
        $capacities_hash{rg_block_capacity} =
           ($capacities_hash{logical_assigned_size} + $capacities_hash{logical_unassigned_size});
    }

    $capacities_hash{logical_configured_size} = $capacities_hash{logical_total_size} - 
                  ($capacities_hash{logical_admin_size} + $capacities_hash{logical_reptarget_size});

    $capacities_hash{subs_logical_configured_size} = $capacities_hash{subs_logical_total_size} -
                  ($capacities_hash{subs_logical_admin_size} + $capacities_hash{subs_logical_reptarget_size});
}

1;
