#$Id$
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2016 Veritas Technologies LLC.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# VERITAS TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
#
# The  Licensed  Software  and  Documentation  are  deemed  to be commercial
# computer  software  as  defined  in  FAR  12.212 and subject to restricted
# rights  as  defined in FAR Section 52.227-19 "Commercial Computer Software
# - Restricted  Rights"  and  DFARS 227.7202,  et seq.  "Commercial Computer
# Software  and  Commercial Computer Software Documentation," as applicable,
# and any successor regulations, whether delivered by Veritas as on premises
# or  hosted  services.  Any  use,  modification,  reproduction  release,
# performance,  display  or  disclosure  of  the  Licensed  Software  and
# Documentation by the U.S. Government shall be solely in accordance with
# the terms of this Agreement. $

package VRTS::Enclosure::Discover::NetApp;
use strict;
use Carp;
require Exporter;
require XML::SAX::ExpatXS;
require VRTS::XMLHandler;
require VRTS::Enclosure::VDID;
require VRTS::Enclosure::Utils;
use Data::Dumper;
require VRTS::Paths;
require VRTS::Schedule;
require File::Spec;
use VRTS::AddOnInfo;

my $vardir = VRTS::Paths::get_path("AppDir");
my ($creds, $probe_timeout, $xml, $vdid_obj, $xml_handler, $enc_hash, $ontap_major_version, $ontap_minor_version);
our $ip_addresses;

my %capabilities = ("VOL_CAPABLE" => 1, "SHOW_RG_LDEV" => 0, "SHOW_RG_PDEV" => 1, "SHOW_PDEV" => 1, "SHOW_LDEV" => 1,
   "SHOW_REPLICATION" => 0, "SHOW_TP_SOURCELDEV" => 1, "SHOW_TP_THINLDEV" => 0, "SHOW_LDEV_PDEV" => 1, "LDEV_PDEV_MAP" => 0, "SHOW_NAS_FS_REPLICATION" => 1,
   "PRIMARY_REMOTE_REP_CAPABLE" => 0, "SHOW_VSET" => 0, "SHOW_TP" => 0, "SHOW_RG" => 1, "SECONDARY_REMOTE_REP_CAPABLE" => 0,
   "SHOW_PDEV_FOR_THIN_LDEV" => 1, "SHOW_PDEV_FOR_TP_SOURCE_THINLDEV" => 0, "PRIMARY_REMOTE_REP_CAPABLE" => 0, "SHOW_QTREE_FILE_CAP" => 0,
   "SHOW_DEVICE_GROUP_REPLICATION" => 1, "STATS_CAPABLE" => 1);

my %mgmt_cap_hash = ('httpd.admin.enable' => 'off', 'httpd.enable' => 'off');

sub new 
{
    my $class = shift;
    my $self = {};

    bless ($self, $class);
    return $self;
}
# Initialize the Netapp discovery
sub initialize
{
    my $top = shift;

    my $module = new VRTS::Enclosure::Discover::NetApp;
    return $module;

}

#
# Get the details about the enclosure
sub get_enclosure_listing
{
    my $self = shift;
    $creds = shift;
    my $ch = $creds->{'control-host'};
    $probe_timeout = $self->{"probe_timeout"};

    my $clipath = VRTS::Paths::get_path("MHBin");

    VRTS::Logger::log('trace', "control-host = [$ch]"); 
    VRTS::Logger::log('trace', "ID = [$creds->{'ID'}]"); 
    VRTS::Logger::log('trace', "server = [$creds->{'server'}]"); 
    VRTS::Logger::log('trace', "port = [$creds->{'port'}]"); 
    VRTS::Logger::log('trace', "user = [$creds->{'user'}]"); 

    my $info = new VRTS::AddOnInfo;
    my $ret = $info->initialize();
    my $installed_version = undef;
    if($ret == 0) {
        my %feature_info = %{$info->get_features()};
        for my $feature (keys %feature_info) {
            next if ($feature_info{$feature}{'ID'} !~ /^VRTSsfmsi-/);
            if ($feature_info{$feature}{'STATUS'} eq "installed" or 
                 $feature_info{$feature}{'STATUS'} eq "enabled")
            {
                 $installed_version = $feature_info{$feature}{'ID'};
            }
        }
    }
    VRTS::Logger::log('debug', "NetApp: The addon installed version is [$installed_version]");

    my $addon_dir = VRTS::Paths::get_path("DeployStoreDir");
    $addon_dir = File::Spec->catfile($addon_dir , $installed_version);
    VRTS::Logger::log('debug', "NetApp: Addon dir is [$addon_dir]");


    $xml_handler = VRTS::XMLHandler->new();
    $xml = XML::SAX::ExpatXS->new(Handler => $xml_handler);

    my $xprtlcmd;
    if ($^O eq "solaris" or $^O eq "linux") {
        $xprtlcmd = "$clipath/xprtlc -l https://localhost/agent/domain/enum | grep '^$ch,'";
    } else {
        $xprtlcmd = "\"$clipath/xprtlc\" -l https://localhost/agent/domain/enum | findstr \"^$ch,\"";
    }

    my $xprtlout = `$xprtlcmd`;

    $xprtlout =~ s/\s+$//;
    my @Domain_info = split(/,/, $xprtlout);
    VRTS::Logger::log('debug', "NetApp: The CH platform is $Domain_info[3].");
    if ($Domain_info[3] !~ /Linux/ && $Domain_info[3] !~ /SunOS/ && $Domain_info[3] !~  /Windows/) {
        VRTS::Logger::log('error', "NetApp: Platform not supported for Netapp discovery");
        return undef, &VRTS::Enclosure::Discover::PLATFORM_UNSUPPORTED;
    }
    if($creds->{'pass'} eq "") {
         VRTS::Logger::log('error', "NetApp: Password is empty");
         return undef, &VRTS::Enclosure::Discover::INCORRECT_PASSWORD;
       
    } 

    my ($source, $out);
    if ($Domain_info[3] =~ /Linux/ ) {
        $source = File::Spec->catfile($addon_dir, "arraycli", "arraycli.linux");
        $out = VRTS::Enclosure::Utils::push($ch, $source, "\$VARDIR\/arraycli", $probe_timeout);
        
    }
    if ($Domain_info[3] =~ /SunOS/ ) {
        $source = File::Spec->catfile($addon_dir, "arraycli", "arraycli.sol");
        $out = VRTS::Enclosure::Utils::push($ch, $source, "\$VARDIR\/arraycli", $probe_timeout);
    }
    if ($out->[0] != 0) {
        my $err = $out->[0] >> 8;
        if ($err == 192) {
            VRTS::Logger::log('error', "Request timed out due to no activity for timeout period");
            return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
        }
    }
   if ($Domain_info[3] =~ /Windows/ ) {
        $source = File::Spec->catfile($addon_dir, "arraycli", "arraycli.win");
        $out = VRTS::Enclosure::Utils::push($ch, $source, "\$VARDIR\/arraycli.exe", $probe_timeout);
        if ($out->[0] != 0) {
            my $err = $out->[0] >> 8;
            if ($err == 192) {
                VRTS::Logger::log('error', "Request timed out due to no activity for timeout period");
                return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
            }
        }
        $source = File::Spec->catfile($addon_dir, "arraycli", "libeay32.dll");
        $out = VRTS::Enclosure::Utils::push($ch, $source, "\$VARDIR\/libeay32.dll", $probe_timeout);
        if ($out->[0] != 0) {
            my $err = $out->[0] >> 8;
            if ($err == 192) {
                VRTS::Logger::log('error', "Request timed out due to no activity for timeout period");
                return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
            }
        }

        $source = File::Spec->catfile($addon_dir, "arraycli", "ssleay32.dll");
        $out = VRTS::Enclosure::Utils::push($ch, $source, "\$VARDIR\/ssleay32.dll", $probe_timeout);
        if ($out->[0] != 0) {
            my $err = $out->[0] >> 8;
            if ($err == 192) {
                VRTS::Logger::log('error', "Request timed out due to no activity for timeout period");
                return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
            }
        }

    }

    my $cli_script = "$clipath/setup_arraycli_perms.pl";
    $out = VRTS::Enclosure::Utils::push_exec($ch,$cli_script,"\$VARDIR\/setup_arraycli_perms.pl", $probe_timeout);
    if ($out->[0] != 0) {
        my $err = $out->[0] >> 8;
        if ($err == 192) {
            VRTS::Logger::log('error', "Request timed out due to no activity for timeout period");
            return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
        }
    }

    $cli_script = "$clipath/setup_arraycli.pl";
    $out = VRTS::Enclosure::Utils::push_exec($ch,$cli_script,"\$VARDIR\/setup_arraycli.pl", $probe_timeout);
    if($out->[0] != 0) {
        my $err = $out->[0] >> 8;
        if ($err == 192) {
            VRTS::Logger::log('error', "Request timed out due to no activity for timeout period");
            return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
        }
        VRTS::Logger::log('error', "NetApp: Error in running $cli_script");
    }
    my $out = execute_api('system-get-info');

    if ($out->[0] != 0)
    {
        my $err = $out->[0] >> 8;
        if ($err == 192) {
            VRTS::Logger::log('error', "Request timed out due to no activity for timeout period");
            return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
        }
         VRTS::Logger::log('error', "NetApp: get_enclosure_listing: Error in getting enclosure listing");
         return undef, &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }

    if ($out->[1] =~ /Error in API invoke/ or $out->[1] =~ /connect to host/ or $out->[1] =~ /not specified/) {
        # Wrong array credential or wrong IP address
        VRTS::Logger::log('error', "NetApp: The array credentials/port or IP address is not correct. Check the creds, IP or Port.");
        return undef, &VRTS::Enclosure::Discover::ARRAY_AUTHORIZATION_FAILED;
    }

    $vdid_obj = new VRTS::Enclosure::VDID ();
    $vdid_obj->initialize("NetApp", 0);
     
    my %markers = ('system-info' => '');
    $xml_handler->initialize(undef, \%markers, 'VRTS::Enclosure::Discover::NetApp::discover_enc_attributes');
    my %parser_args = (Source => {SystemId => $out->[1]});
    eval{$xml->parse(%parser_args)};
    if($@) {
        VRTS::Logger::log('debug', 'Unable to parse XML for discover_enc_attributes');
        return undef, &VRTS::Enclosure::Discover::PROBE_INTERNAL_ERROR;
    }

    #Check Licenses only on IMS
    if(VRTS::Util::is_ims() && check_license()) {
        VRTS::Logger::log('error', 
            'Please enable the settings httpd.admin.enable, httpd.enable on the filer and configure it again');
        return undef, &VRTS::Enclosure::Discover::NETAPP_LICENSE_ERROR;
    }

    return $enc_hash;
}

sub execute_api {
    my $api = shift;
    my $script = File::Spec->catfile( $vardir, 'netapp.pl');
    _create_probe_script($creds->{'ID'}, $creds->{'server'}, $creds->{'user'}, 
        $creds->{'pass'}, $creds->{'port'}, $api, $script);

    my $out = VRTS::Enclosure::Utils::push_exec($creds->{'control-host'}, $script, "\$VARDIR\/netapp.pl", 
        $probe_timeout, 2, 1); 
    unlink($script);

    if ($out->[2] ne '') {
        VRTS::Logger::log('debug', "CLI execution message = [$out->[2]]");
    }  

    VRTS::Logger::log('trace', "COMMAND = [$api]"); 
    VRTS::Logger::log('trace', "OUTPUT = [$out->[1]]"); 

    return $out;
}

sub _create_probe_script
{
    my ($id, $server, $user, $pass, 
        $port, $api, $script, $vfiler_name) = @_;

    if (not defined $vfiler_name or $vfiler_name eq "")
    {
        $vfiler_name = "vfiler0";
    }

    my $r = open(my $fh, '>', $script);
    if (not defined($fh)) {
        VRTS::Logger::log('error', "Unable to open file $script for writing");
        exit $r;
    }

    binmode $fh;
    print $fh <<EOF;
#!/opt/VRTSsfmh/bin/perl
BEGIN { \@INC = ('/opt/VRTSsfmh/lib/modules') if ( \$^O !~ /Win32/i ) }
require VRTS::Paths;
require VRTS::Util;
my \$vardir = VRTS::Paths::get_path('AppDir');
my \$bindir = VRTS::Paths::get_path('MHBin');
my (\$rc, \$pass) = VRTS::Util::decrypt_encoded_passwd('$pass', '$id');
if (\$rc) {
    print "decrypt_encoded_passwd failed with error code = [\$rc]";
    exit \$rc;
}
my \$cmd;
if ( \$^O !~ /Win32/i ) {
    \$cmd = "\$bindir/arraycli"; 
} else {
    \$cmd = "\$bindir\\\\arraycli.exe";
    \$pass = VRTS::Util::esc_special_chars(\$pass);
}
if (! -x \$cmd) {
    print 'not found'; 
    return;
}
system(\$cmd, '$server', '$user', \$pass, '$port', '$vfiler_name', 'Netapp', '$api');
EOF
    close($fh);
}

sub discover_enc_attributes {
    my $handler = shift;

    my %val_hash;

    my $vdid = $vdid_obj->get_enclosure_vdid("NETAPP",undef,
                                 $handler->{DATA}->{'system-info'}->{'system-serial-number'});
    my $old_vdid = $vdid_obj->get_enclosure_vdid("NETAPP",undef,
                                 $handler->{DATA}->{'system-info'}->{'system-id'});
    $val_hash{VDID} = $vdid;
    $val_hash{OLDVDID} = $old_vdid;

    $val_hash{serial} = $handler->{DATA}->{'system-info'}->{'system-serial-number'};
    $val_hash{model} = $handler->{DATA}->{'system-info'}->{'system-model'};
    $val_hash{displayname} = $handler->{DATA}->{'system-info'}->{'system-name'};
    $val_hash{product} = $handler->{DATA}->{'system-info'}->{'system-model'};
    $val_hash{vendor} = 'NetApp';
    $val_hash{'system-id'} = $handler->{DATA}->{'system-info'}->{'system-id'};

    $enc_hash->{$vdid} = \%val_hash;
}

sub provide_capabilities
{
    my $self = shift;

    return \%capabilities;
}

sub check_license {
    _discover_ontap_api_version();

    if ((not defined($ontap_major_version))|| (not defined($ontap_minor_version))
        || (($ontap_major_version < 1) && ($ontap_minor_version < 3))) {
        VRTS::Logger::log('error', 'Ontapi version not supported to get license details');
        return 1;
    }

    discover_mgmt_capabilities();
    return 1 if (($mgmt_cap_hash{'httpd.admin.enable'} eq 'off') 
        || ($mgmt_cap_hash{'httpd.enable'} eq 'off')); 

    #Check if we are able to discover IP Addresses 
    discover_ips();
    if((not defined($ip_addresses)) || 
        ((defined($ip_addresses)) && (scalar(@{$ip_addresses}) == 0))) {
        VRTS::Logger::log('error', 'Could not get IP addresses');
        return 1;
    }
    VRTS::Logger::log('debug', 
        'Options httpd.admin.enable, httpd.enable are enabled and IP addresses are also discoverable');

    return 0;
}

sub discover_ips {
    my $out = execute_api('net-ifconfig-get');
    if ($out->[0] != 0) {
        VRTS::Logger::log('error', 'discover_ips: Error in getting IP Addresses');
        unlink $out->[1] if (-f $out->[1]);
    }

    my %markers = ('v4-primary-address' => '');
    $xml_handler->initialize(undef, \%markers, 'VRTS::Enclosure::Discover::NetApp::_get_ip_addresses');
    my %parser_args = (Source => {SystemId => $out->[1]});
    eval{$xml->parse(%parser_args)};
    if($@) {
        VRTS::Logger::log('error', 'Unable to parse XML for _get_ip_addresses');
    }
}

sub discover_mgmt_capabilities {   
    my $out = execute_api('options-list-info');
    if ($out->[0] != 0) {
        VRTS::Logger::log('error', 
                     "discover_mgmt_capabilities: Error in getting management
                     capability details for filer [$creds->{'server'}]");
        unlink $out->[1] if (-f $out->[1]);
    }

    my %markers = ('option-info' => '');
    $xml_handler->initialize(undef, \%markers, 'VRTS::Enclosure::Discover::NetApp::_get_mgmt_capabilities');
    my %parser_args = (Source => {SystemId => $out->[1]});
    eval{$xml->parse(%parser_args)};
    if($@) {
        VRTS::Logger::log('error', 'Unable to parse XML for _get_mgmt_capabilities');
    }
}

sub _discover_ontap_api_version {
    my $out = execute_api('system-get-ontapi-version');
    if ($out->[0] != 0) {
        VRTS::Logger::log('error', '_discover_ontap_api_version: Cant get ontap API version');
        unlink $out->[1] if (-f $out->[1]);
    }

    my %markers = ('major-version' => '', 'minor-version' => '');
    $xml_handler->initialize(undef, \%markers, "VRTS::Enclosure::Discover::NetApp::_get_ontap_version");
    my %parser_args = (Source => {SystemId => $out->[1]});

    eval{$xml->parse(%parser_args)};
    if($@) {
        VRTS::Logger::log('error', 'Unable to parse XML for ontap version');
    }
}


sub _get_ontap_version {
    my ($handler, $token) = @_;

    if ($token eq 'major-version') {
        $ontap_major_version = $handler->{DATA}->{'major-version'}->{'major-version'};
    } elsif ($token eq 'minor-version') {
        $ontap_minor_version = $handler->{DATA}->{'minor-version'}->{'minor-version'};
    }
}

sub _get_mgmt_capabilities {
    my $handler = shift;

    if (exists($handler->{'DATA'}->{'option-info'}->{'name'})) {
        my $option = $handler->{'DATA'}->{'option-info'}->{'name'};
        if ((defined($option)) && ($option ne '') && 
            (exists($mgmt_cap_hash{$option}))) {
            $mgmt_cap_hash{$option} = $handler->{'DATA'}->{'option-info'}->{'value'};
        }
    }
}

sub _get_ip_addresses {
    my $handler = shift;
    push (@{$ip_addresses}, $handler->{'DATA'}->{'v4-primary-address'}->{'address'});
}

1;
