#$Copyright$Id

package VRTS::Enclosure::Discover::RecoverPoint;
use strict;
require VRTS::Enclosure::VDID;
require VRTS::Enclosure::Utils;

my $probe_timeout;

sub new 
{
    my $class = shift;
    my $self = {};
    bless ($self, $class);
    return $self;
}

sub initialize
{
    my $top = shift;
    my $array_module = new VRTS::Enclosure::Discover::RecoverPoint;
    return $array_module;
}

sub provide_capabilities
{
    return undef;
}

##
# Retrieves information about the RPA.
# 
sub get_enclosure_listing
{
    my ($self, $params) = @_;
    my $enc_hash;
    my $cluster_data;
    my $ch = $params->{"control-host"};
    my $cluster_server = $params->{"server"};
    my $user = $params->{"user"};
    my $id = $params->{ID};
    my $pass = $params->{"pass"};
    my $secfile = $params->{"pem-file"};
    $probe_timeout = $self->{"probe_timeout"};
    
    VRTS::Logger::log ("trace", "get_enclosure_listing(): CH: $ch, Server: $cluster_server, User: $user"); 

    my $ch_version = VRTS::Enclosure::Utils::get_mh_version($ch);
    if (!defined($ch_version)) {
        VRTS::Logger::log ("error", "Discovery host may be down.");
        return undef, &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    } 
    $ch_version =~ /(\d+)\S+/;
    $ch_version =~ /(\d+).(\d+)./;
    my $main = $1;
    my $minor = $2; 
    my $version = "$main".".$minor";
    VRTS::Logger::log ("trace", "Discovery host version $ch_version($version) detected.");
    if ($version < 7.1) {
        VRTS::Logger::log ("error", "Discovery host version $version not supported. Please specify discovery host having MH Version 7.1 and above");
        return undef, &VRTS::Enclosure::Discover::RPA_MH_VERSION_NOT_SUPPORTED;
    }

    #First try with password. 
    my $pw_opt = '-pw';
    my $probecmd = 'get_local_cluster; get_system_settings';
    my $out = exec_rp_probe_script($ch, $user, $id, $pw_opt, $pass, $cluster_server, $probecmd);
    if ($out->[0] != 0) {
        my $err = $out->[0] >> 8;
        if ($err == 192) {
            VRTS::Logger::log('error', "Request timed out due to no activity for timeout period");
            return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
        }
    }

    VRTS::Logger::log('trace', 'COMMAND = [\'get_local_cluster; get_system_settings\'] with password'); 
    VRTS::Logger::log('trace', "OUTPUT = [$out->[1]]"); 

    my $err_msg;
    # Parse the command O/P
    if ($out->[1] =~ /Connection timed out|connection refused/is) {
        VRTS::Logger::log ("error", "Wrong IP Address provided or Array is unreachable");
        return undef, &VRTS::Enclosure::Discover::NO_CONNECTION_INVALID_IP;
    } elsif ($out->[1] =~ /Authentication failure, invalid credentials/is) {
        VRTS::Logger::log ("error", "Wrong username or password");
        $err_msg = $out->[1];
        if($secfile eq "") {
            return undef, &VRTS::Enclosure::Discover::AUTHENTICATION_FAILED;
        }
    } elsif ($out->[1] =~ /FATAL ERROR:\s*(.*)/is) {
        VRTS::Logger::log ("error", "$1");
        $err_msg = $out->[1];
    } elsif ($out->[1] =~ /decrypt_encoded_passwd failed/is) {
        VRTS::Logger::log ("error", "$1");
        $err_msg = $out->[1];
    }#end 
    
    my @alines = split(/\n/, $out->[1]);
    my $rpa_cluster_name = "";
    my $rpa_serial_id = "";
    my $rpa_type = "";
    my $flag_cluster_name = 0;
    foreach (@alines)
    {
        chomp;
        my ($rpa_attr, $rpa_id) = split (/:/);
	if ($rpa_attr =~ /Local Cluster Name/) {  
		#Look for Local cluster name
		$rpa_id=~s/^\s+//;
		$rpa_cluster_name = $rpa_id;
		next;
	}
	#Search for Software Serial id.
	#itarate till cluster name. else we may end up serching remote cluster name serial id
 	if ($rpa_attr =~ /$rpa_cluster_name/) {
		VRTS::Logger::log ("trace", "out1 :: $rpa_attr :: $rpa_id :: $rpa_cluster_name");
        	$flag_cluster_name = 1;
	}
	next if !$flag_cluster_name;
	
	#next till Software serial ID.
        next if $rpa_attr !~ /Software serial ID/; #Serial id
        if (not defined($rpa_id) || $rpa_id eq "") {
            VRTS::Logger::log ("error", "get_enclosure_listing(): Failed to get RecoverPoint Software serial ID.");
            return undef, &VRTS::Enclosure::Discover::PROBE_INTERNAL_ERROR;
        }
	$rpa_serial_id =  $rpa_id;
	last;
    }
        
    VRTS::Logger::log ("debug", "get_enclosure_listing(): Found RPA Cluster Name : $rpa_cluster_name, Software serial ID : $rpa_serial_id");
    if ($rpa_cluster_name eq "" || $rpa_serial_id eq "") {
        VRTS::Logger::log ("error", "get_enclosure_listing(): Failed to get Local Cluster name or RecoverPoint Software serial ID.");
        return undef, &VRTS::Enclosure::Discover::RPA_SERIAL_ID_NOT_FOUND_ERROR;
    }

	#Find RPA type 
    	$probecmd = "get_licenses cluster=$rpa_cluster_name aggregated=yes";
        VRTS::Logger::log ("trace", "get_enclosure_listing(): probecmd : $probecmd");
    	my $out = exec_rp_probe_script($ch, $user, $id, $pw_opt, $pass, $cluster_server, $probecmd);
    	if ($out->[0] != 0) {
        	my $err = $out->[0] >> 8;
        	if ($err == 192) {
            	VRTS::Logger::log('error', "Request timed out due to no activity for timeout period");
            	return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
        	}
    	}

    	VRTS::Logger::log('trace', 'COMMAND = [\'get_licenses cluster=$rpa_cluster_name aggregated=yes\'] with password'); 
    	VRTS::Logger::log('trace', "OUTPUT = [$out->[1]]"); 

    	my $err_msg;
    	# Parse the command O/P
   	 if ($out->[1] =~ /Connection timed out|connection refused/is) {
        	VRTS::Logger::log ("error", "Wrong IP Address provided or Array is unreachable");
        	return undef, &VRTS::Enclosure::Discover::NO_CONNECTION_INVALID_IP;
    	} elsif ($out->[1] =~ /Authentication failure, invalid credentials/is) {
        	VRTS::Logger::log ("error", "Wrong username or password");
        	$err_msg = $out->[1];
        	if($secfile eq "") {
            		return undef, &VRTS::Enclosure::Discover::AUTHENTICATION_FAILED;
        	}
    	} elsif ($out->[1] =~ /FATAL ERROR:\s*(.*)/is) {
        	VRTS::Logger::log ("error", "$1");
        	$err_msg = $out->[1];
    	} elsif ($out->[1] =~ /decrypt_encoded_passwd failed/is) {
        	VRTS::Logger::log ("error", "$1");
        	$err_msg = $out->[1];
    	}#end 
	
    	@alines = split(/\n/, $out->[1]);
	my $rpa_type; 
    	foreach (@alines)
    	{
        	chomp;
        	my ($rpa_attr, $rpa_id) = split (/:/);
		if ($rpa_attr =~ /Type/) {  
			#Look for Local cluster name
			$rpa_id=~s/^\s+//;
			$rpa_type = "RecoverPoint\\" . "$rpa_id";
        		VRTS::Logger::log ("debug", "get_enclosure_listing(): Found RPA Cluster type : $rpa_type");
			last;	
		}
	}

    if ($rpa_type eq "") {
        VRTS::Logger::log ("error", "get_enclosure_listing(): Failed to get Local Cluster name or RecoverPoint Software serial ID.");
        return undef, &VRTS::Enclosure::Discover::RPA_TYPE_NOT_FOUND_ERROR;
    }
	
    # Form hash of cluster attributes from the command O/P
    $enc_hash = _get_enclosure_details ($rpa_cluster_name, $rpa_serial_id, $rpa_type);

    if (not defined($enc_hash)) {
        return undef, &VRTS::Enclosure::Discover::PROBE_INTERNAL_ERROR;
    }
    else {
        return $enc_hash, &VRTS::Enclosure::Discover::PROBE_SUCCESS;
    }
}

#
#  Creates probe script to be executed on the RPA .
#
sub _create_probe_script
{
    my ($user, $id, $pw_opt, $pass, $cluster_server, $probecmd) = @_;
    my $script = File::Spec->catfile(VRTS::Paths::get_path("TempDir"),"RecoverPoint.halssh.$$.$cluster_server.pl");
    my $fh;
    my $r = open($fh, '>', $script);
    if (not defined($fh)) {
        VRTS::Logger::log('error', "Unable to open file $script for writing");
        exit $r;
    }
   binmode $fh;
   VRTS::Logger::log('trace', "Options :: $user :: $pw_opt :: $pass :: $cluster_server :: $probecmd::$id"); 
    print $fh <<EOF;
#!/opt/VRTSsfmh/bin/perl
BEGIN { \@INC = ('/opt/VRTSsfmh/lib/modules') if ( \$^O !~ /Win32/i ) }
require VRTS::Util;
\$ENV{'LD_LIBRARY_PATH'} = '/opt/VRTSsfmh/lib' if ( \$^O !~ /Win32/i );
my \$cmd = File::Spec->catfile(VRTS::Paths::get_path('MHBin'), 'halssh');
my \$pw_opt = '$pw_opt';
my \$pass = '$pass';
if (\$pw_opt eq '-pw') {
    my \$rc;
    (\$rc, \$pass) = VRTS::Util::decrypt_encoded_passwd(\$pass, '$id');
    if (\$rc) {
        print "decrypt_encoded_passwd failed with error code = [\$rc]";
        exit \$rc;
    }
    if ( \$^O =~ /Win32/i ) {
        \$pass = VRTS::Util::esc_special_chars(\$pass);
    }
} 
if ( \$^O =~ /Win32/i ) {
    \$cmd = \$cmd . '.exe';
}

if (! -x \$cmd) {
    print 'not found'; 
    return;
}

system(\$cmd, '-ssh', '-l', '$user', \$pw_opt, \$pass, '-4', '$cluster_server', '$probecmd');
EOF
    return $script 
}

##
#  Returns a hash of enclosure details.
#
sub _get_enclosure_details
{
    my ($rpa_cluster_name, $rpa_id, $rpa_type) = @_;

    my $vdid_obj = new VRTS::Enclosure::VDID ();
    my $array_vendor = "EMC";
    my $array_model = $rpa_type;
    my $array_family = "RecoverPoint";
    $vdid_obj->initialize($array_family, 0);
    my $enc_hash;
    my $enc_detail = {};

    $rpa_id=~s/^\s+//;
    $rpa_id = lc ($rpa_id);
    my $e_vdid = $vdid_obj->get_enclosure_vdid($array_vendor, $array_family, $rpa_id);
    $enc_detail->{VDID} = $e_vdid;
    $enc_detail->{serial} = $rpa_id;
    $enc_detail->{displayname} = $rpa_cluster_name;
    $enc_detail->{id} = $e_vdid;
    $enc_detail->{product} = $array_family;
    $enc_detail->{vendor} = $array_vendor;
    $enc_detail->{model} = $array_model;
    $enc_hash->{$e_vdid} = $enc_detail;
    return $enc_hash;
}

# This method creates the wrapper perl script to be run on a control host for 
# RPA discovery. The script contains code to run the RPA commands over ssh
# on the remote RPA.
sub create_discovery_script
{
    my ($cluster, $user, $id, $pass, $secfile, $g_discover_shell_script)  = @_;

    my $perl_script = File::Spec->catfile(VRTS::Paths::get_path("TempDir"), "RPA_halssh_".$$."_$cluster.pl");
    my $PERLSC = open(my $fh, '>', $perl_script);
    if (! $PERLSC)
    {
        VRTS::Logger::log ("error", "_get_create_discovery_script(): Failed to open file $perl_script for writing: $@");
        return undef;
    }

    # The script shall be constructed in 4 parts:
    # (1) The initial code containing module includes, ssh script file creation, and beginning of ssh script 
    # variable definition
    # (2) The ssh script itself, as defined in $g_discover_shell_script variable.
    # (3) The end of ssh script section, and variable initializations, which shall require expansion of variables
    # passed to this function
    # (4) The final code which includes invoking the script via halssh module.

    my $perl_script1 = <<'PERLSC1';
#!/opt/VRTSsfmh/bin/perl
BEGIN { @INC = ('/opt/VRTSsfmh/lib/modules') if ( $^O !~ /Win32/i ) }

require VRTS::Paths;
require File::Spec;
require VRTS::Util;

my $g_discover_shell_script = <<'SHELLSC';
PERLSC1

# The shell script is appended to the above code - see the end part of this method.

    my $perl_script2 = <<PERLSC2;
SHELLSC

my (\$cluster, \$user, \$id, \$pass, \$secfile)  = (\'$cluster\', \'$user\', \'$id\', \'$pass\', \'$secfile\');
PERLSC2

    my $perl_script3 =<<'PERLSC3';

my $cmd = VRTS::Paths::get_path('MHBin');
$cmd = File::Spec->catfile($cmd, 'halssh');

my $ssh_script = File::Spec->catfile(VRTS::Paths::get_path('TempDir'), "RecoverPoint_discover_$$_$cluster.sh");
my $fh;
if (!open($fh, '>', $ssh_script)) {
    return;
}

binmode $fh;
print $fh $g_discover_shell_script;
close ($fh);

if ($^O !~ /Win32/i ) {
    $ENV{'LD_LIBRARY_PATH'} = '/opt/VRTSsfmh/lib';
} else {
    $cmd = $cmd . '.exe';
}
if (! -x $cmd) {
    print 'not found'; 
    return;
}

my @cmds;
push(@cmds, $cmd); push(@cmds, '-ssh');
push(@cmds, '-l'); push(@cmds, $user);
if ((defined($secfile)) && ($secfile ne '')) {
    push(@cmds, '-i'); push(@cmds, $secfile);
} elsif ((defined($pass)) && ($pass ne '')) {
    my $rc;
    ($rc, $pass) = VRTS::Util::decrypt_encoded_passwd($pass, $id);
    if ($rc) {
        print "decrypt_encoded_passwd failed with error code = [$rc]";
        return $rc;
    }
    if ($^O =~ /Win32/i ) {
        $pass = VRTS::Util::esc_special_chars($pass);
    }
    push(@cmds, '-pw'); push(@cmds, $pass);
} 
push(@cmds, '-4'); push(@cmds, $cluster);
push(@cmds, '-m'); push(@cmds, $ssh_script);
system(@cmds);
        
PERLSC3

    my $script = $perl_script1 . $g_discover_shell_script . $perl_script2 . $perl_script3;
    binmode $fh;
    print $fh $script; 
    close ($fh);
    return $perl_script;
}

sub exec_rp_probe_script
{
    my ($ch, $user, $id, $pw_opt, $pass, $cluster_server, $probecmd) = @_;
    # Create script for probing the RPA.
    my $script = _create_probe_script($user, $id, $pw_opt, $pass, $cluster_server, $probecmd);
    if (not defined ($script)) {
        VRTS::Logger::log ("error", "Error creating script");
        return undef, &VRTS::Enclosure::Discover::PROBE_INTERNAL_ERROR;
    }

    # Run the script over ssh on the RPA.
    my $out = VRTS::Enclosure::Utils::push_exec ($ch, $script, '$VARDIR/RPA.halssh.'."$$.$cluster_server.pl", $probe_timeout);
    unlink ($script);
    return $out;
}
1;
