#$Id$
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2016 Veritas Technologies LLC.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# VERITAS TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
#
# The  Licensed  Software  and  Documentation  are  deemed  to be commercial
# computer  software  as  defined  in  FAR  12.212 and subject to restricted
# rights  as  defined in FAR Section 52.227-19 "Commercial Computer Software
# - Restricted  Rights"  and  DFARS 227.7202,  et seq.  "Commercial Computer
# Software  and  Commercial Computer Software Documentation," as applicable,
# and any successor regulations, whether delivered by Veritas as on premises
# or  hosted  services.  Any  use,  modification,  reproduction  release,
# performance,  display  or  disclosure  of  the  Licensed  Software  and
# Documentation by the U.S. Government shall be solely in accordance with
# the terms of this Agreement. $

# 
# Probe script for VNX: This will work for VNX filer, VNX block and VNX unified
# device. For File, it will execute XML API to get the file info. And for VNX
# block, it will execute naviseccli command to get block info. If its VNX
# unified, it will execute both the commands. Once done it will form different
# enclosures for the devices. So basically VNX will work as device manager which
# has two different arrays, File and Block.
#
package VRTS::Enclosure::Discover::VNX;
BEGIN {push @INC, "/opt/VRTSperl/lib/site_perl/5.10.0/"};
use strict;
use Carp;
require Exporter;
require XML::Simple;
require VRTS::Paths;
require VRTS::Enclosure::Utils;
require XML::SAX::ExpatXS;
require VRTS::XMLHandler;


my $ch;
my $ch1;
my $vardir;
my $enchash;
my $xml;
my $err_no = undef;
my $cli_path = undef;
my $model = undef;
my $macro_version = 6.29;
my $probe_timeout;


my %capabilities = ("VOL_CAPABLE" => 0, "SHOW_RG_LDEV" => 1, "SHOW_RG_PDEV" => 1, "SHOW_PDEV" => 1, "SHOW_LDEV" => 1,
    "SHOW_REPLICATION" => 1, "SHOW_TP_SOURCELDEV" => 0, "SHOW_TP_THINLDEV" => 1, "SHOW_LDEV_PDEV" => 1, "LDEV_PDEV_MAP" => 0,
    "PRIMARY_REMOTE_REP_CAPABLE" => 1, "SHOW_VSET" => 0, "SHOW_TP" => 1, "SHOW_RG" => 1, "SECONDARY_REMOTE_REP_CAPABLE" => 1,
    "SHOW_PDEV_FOR_THIN_LDEV" => 0, "SHOW_PDEV_FOR_TP_SOURCE_THINLDEV" => 0,
    "SHOW_DEVICE_GROUP_REPLICATION" => 0, "STATS_CAPABLE" => 1, "SHOW_NODE" => 1, "SHOW_NODE_CAPACITY" => 1);

# Map for Scoping details
my %scope_map = ("Global" => "0", "Local" => "1", "Ldap" => "2");

sub new 
{
    my $class = shift;
    my $self = {};

    bless ($self, $class);
    return $self;
}

sub initialize
{
    my $top = shift;

    my $module = new VRTS::Enclosure::Discover::VNX;
    $vardir= VRTS::Paths::get_path("AppDir");
    return $module;

}

sub get_enclosure_listing
{
    my $self = shift;
    my $params = shift;
    my $server = $params->{"server"};
    my $block_server = $params->{"ARRAY_VNX_IP_BLOCK"};
    $probe_timeout = $self->{"probe_timeout"};
    my ($cli_op, $ret) = undef; 
    
    VRTS::Logger::log('trace', "control-host = [$params->{'control-host'}]"); 
    VRTS::Logger::log('trace', "ID = [$params->{ID}]"); 
    VRTS::Logger::log('trace', "server = [$server]"); 
    VRTS::Logger::log('trace', "block server = [$block_server]"); 
    VRTS::Logger::log('trace', "user = [$params->{'user'}]"); 
    VRTS::Logger::log('trace', "port = [$params->{'port'}]"); 

    if($server ne '' ) {
        ($xml, $ret) = get_filer_enclosure_listing($params);
        if(defined $ret) {
            return undef, $ret;
        }
    }
    if($block_server ne '') {
        ($cli_op, $ret) = get_block_enclosure_listing($params);
        if (not defined $cli_op) {
            return (undef, $ret);
        }
    } else {
        $capabilities{'STATS_CAPABLE'} = 0;
        #As $block_server is not definded, we shoud now report
        #STATS_CAPABLE 0, So metering can not be enabled for
        #VNX array.
    }
    if($server eq '' and $block_server eq '') 
    {
        VRTS::Logger::log('error', 'No enclosure IP/Name is specified for configuration');
        return undef, &VRTS::Enclosure::Discover::VNX_BOTH_ENCLOSURE_FIELDS_EMPTY;
    }
    fill_enc_details($xml->{Response}, $cli_op, $server);
    return $enchash;

}

# Get enclosure info for VNX file
sub get_filer_enclosure_listing
{
    my $params = shift;

    $ch = $params->{"control-host"};
    $ch1 = $params->{"secondary-ch"};
    my $id = $params->{ID};
    my $server = $params->{"server"};
    my $user = $params->{"user"};
    my $pass = $params->{"pass"};
    my ($out, $ret) = undef;
        
    #### Get Enclosure Capabilities
    $out = _get_array_capabilities($ch,$id,$server,$user,$pass);
    if ($out->[0] != 0)
    {
        my $err = $out->[0] >> 8;
        if ($err == 192)
        {
            print "Request timed out due to no activity for timeout period\n";
            return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
        }
    }
    if ($out->[1] eq "" || $out->[1] =~ /Server is not/)
    {
        VRTS::Logger::log('error', "VNX:Unable to contact VNX Server from primary.");
        return undef,&VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }

##############################################################
# Commenting this part of code. We can revisit exact behavior across all arrays
# in future, if required.
#if (defined ($ch1) && $ch1 ne "")
#       {
#           print STDERR "VNX:Unable to contact VNX from primary, trying from secondary\n";
            # Contact VNX server through secondary control host
#           $out = _get_array_capabilities($ch1,$server,$user,$pass);

#           if ($out->[1] eq "" || $out->[1] =~ /Server is not/)
#           {
#                print STDERR "VNX:Unable to contact VNX Server from primary as well as secondary\n";
#                return undef,&VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
#           }
#       }
#       else
#       {
#           print STDERR "VNX:Unable to contact VNX Server from primary. secondary not present\n";
#           return undef,&VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
#       }
        # Make secondary as primary for further run
#        $ch = $ch1;
#    }
#else
#   {
#       # Even if primary is working fine, if user has provided secondary
        # Try if it is working fine.
#       if (defined ($ch1) && $ch1 ne "")
#       {
#           $out   = _get_array_capabilities($ch1,$server,$user,$pass);
#           if ($out->[1] eq "" || $out->[1] =~ /Server is not/)
#           {
#                print STDERR "VNX:Unable to contact VNX Server secondary\n";
#                return undef,&VRTS::Enclosure::Discover::SEC_CONNECT_FAILED;
#           }
#       }
#   }
#########################################################
    if( ($out->[1] ne "") && ($out->[1] =~ /403 Forbidden/))
    {
    	print STDERR "VNX:Access denied. Error returned [$out->[1]]\n";
    	return undef,&VRTS::Enclosure::Discover::AUTHENTICATION_FAILED;
    }
    if( ($out->[1] ne "") && ( ($out->[1] =~ /400 Bad Request/) ||
         ($out->[1] =~ /Could not connect to server/) ) ) {
        print STDERR " VNX:Bad Request Error returned [$out->[1]]\n";
        return undef, &VRTS::Enclosure::Discover::NO_CONNECTION_INVALID_IP;
    }
    if( ($out->[1] ne "") && ($out->[1] =~ /Service Unavailable/) )
    {
        print STDERR " VNX:XML API Server on Control Station either not running or not responding [$out->[1]]\n";
        VRTS::Logger::log('error',"XML API Server on Control Station either not running or not responding");
        return undef, &VRTS::Enclosure::Discover::XML_API_SERVER_NOT_RESPONDING;
    }

    my $xs = XML::Simple->new();
    $xml  = eval{$xs->XMLin($out->[1])};
    return $xml;
}

# Get enclosure info for VNX block
sub get_block_enclosure_listing
{
    my $params = shift;

    my ($arg1, $arg2);
    my $ch = $params->{"control-host"};
    my $id = $params->{ID};
    my $secfilepath = $params->{"pem-file"};
    my $scope;
    my $port = $params->{"port"};
    my $server = $params->{'ARRAY_VNX_IP_BLOCK'};
    if($server =~ /(.*):\s*(\d+)\s*/) {
       $server = $1;
       $scope = $2;
       if($scope < 0 || $scope > 2) {
            my $msg = "Incorrect Scope $scope specified for VNX block ip $server .";
            VRTS::Logger::log('error', $msg);
            print STDERR $msg;
            return undef, &VRTS::Enclosure::Discover::AUTHENTICATION_FAILED;
       }
    } else {
       $scope = 0;
    }
    $port = "443" if (not defined $port or $port eq "");

    $cli_path = $params->{"install-path"};
    $cli_path = "/opt/Navisphere/bin/" if (not defined $cli_path or $cli_path eq "");

    my $pass = quotemeta($params->{'pass'});
    $pass =~ s/\\/\\\\\\/g;

    my $script1 = "$vardir/navicli.pl";
    _create_probe_script("-help", undef, undef, undef, $script1);

    my $out1 = VRTS::Enclosure::Utils::push_exec($ch, $script1, "\$VARDIR\/navicli.pl", $probe_timeout);

    VRTS::Logger::log('trace', 'COMMAND = [help]'); 
    VRTS::Logger::log('trace', "OUTPUT = [$out1->[1]]"); 
    
    if ($out1->[0] != 0)
    {
        my $err = $out1->[0] >> 8;
        if ($err == 192)
        {
            print "Request timed out due to no activity for timeout period\n";
            return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
        }
         print "Error in push_exec\n";
         return undef, &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }
    if ($out1->[0] == 0 and ( ($out1->[1] eq '') or 
         ($out1->[1] =~ /The system cannot find the path specified/) or
         ($out1->[1] =~ /not found/) or ($out1->[1] =~ /No such file/) or
         ($out1->[1] =~ /syntax error/) or ($out1->[1] =~ /is not recognized as an internal or external command/ ) ) ) {
        print STDERR "VNX::Incorrect CLI location\n";
        return undef, &VRTS::Enclosure::Discover::VENDOR_COMP_NOT_FOUND;
    }


    unlink($script1);

    #When naviseccli is configured at medium security level, 
    #below command will accept certificate permanently.    
    VRTS::Enclosure::Utils::accept_cert_for_array("-h $server -Port $port -User " .
       $params->{'user'}, $id, $pass, "-Scope $scope getagent",
       "$vardir/cert.pl", $cli_path  );
    my $out = VRTS::Enclosure::Utils::push_exec($ch, "$vardir/cert.pl", "\$VARDIR\/cert.pl");
 
    VRTS::Logger::log('trace', 'COMMAND = [getagent]'); 
    VRTS::Logger::log('trace', "OUTPUT = [$out->[1]]"); 

    {
        my (@subject, @cn, %cn_hash); 
        if ( (( $out->[1] =~ /unable\s*to\s*validate\s*the\s*identity\s*of\s*the\s*server/i ) ||  
                ( $out->[1] =~ /cannot\s*confirm\s*that\s*the\s*connection\s*to\s*the\s*server\s*is\s*secure/i ) 
             ) 
        
            && ( @subject = ($out->[1] =~ /(^Subject.*CN=.*)/mg))) {

            if ($#subject > 0 ) {
                VRTS::Logger::log('warning', 'found more than 1 message with Name Subject :  ' . join("\n", @subject) );
            }
            
            @cn = map { ($_ =~ /CN=([^,]+)/g ) } @subject ;
            VRTS::Logger::log('trace', 'Common Names for array ' . join("," , @cn) );
            undef @cn_hash{@cn};
            if (!exists $cn_hash{$server}) {
                 return undef, &VRTS::Enclosure::Discover::ENCLOSURE_CERT_CN_MISMATCH;
            }
        }

    }

    unlink("$vardir/cert.pl"); 

    my $data = $out1->[1];
    $data =~ m/Revision\s*([0-9]+\.[0-9]+)\s*/;

    my $version = $1;
    if ($version < $macro_version)
    {
        # The minimum supported macro version is 6.29
        # for naviseccli  
        return undef, &VRTS::Enclosure::Discover::VENDOR_COMP_VERSION_INCOMPAT;
    }

    if (not defined $secfilepath) 
    {
        $arg1 = "-h $server -Port $port -User $params->{'user'}";
        $arg2 = "-Scope $scope getagent";
    }
    else 
    {
        $secfilepath =~ s/^\s*//;
        $secfilepath =~ s/\s*$//;
        $secfilepath =~ s/\\/\\\\/g;
 
        $arg1 = "-h $server -secfilepath $secfilepath getagent";
        $arg2 = undef;
    }
    
    my $script = "$vardir/navicli.pl"; 
    _create_probe_script($arg1, $id, $pass, $arg2, $script);
    my $out = VRTS::Enclosure::Utils::push_exec($ch, $script, "\$VARDIR\/navicli.pl", $probe_timeout);

    VRTS::Logger::log('trace', 'COMMAND = [xml getagent]'); 
    VRTS::Logger::log('trace', "OUTPUT = [$out->[1]]"); 

    if ($out->[0] != 0)
    {
        my $err = $out->[0] >> 8;
        if ($err == 192)
        {
            print "Request timed out due to no activity for timeout period\n";
            return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
        }
         print "Error in push_exec\n";
         return undef, &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }
      
    unlink($script); 
    if ($out->[1] =~ /not found/ or $out->[1] =~ /No such file/)
    {
        # Naviseccli path not found.
        return undef, &VRTS::Enclosure::Discover::VENDOR_COMP_NOT_FOUND;
    }
    elsif ($out->[1] =~ /timed out/i or $out->[1] =~ /time out/i)
    {
        # No arrays found
        # naviseccli timed out
        return undef, &VRTS::Enclosure::Discover::ARRAY_NOT_REACHABLE;
    }
    elsif ($out->[1] =~ /Authentication failed/i)
    {
        # Authentication failed
        # username or password or scope or security file is incorrect 
        return undef, &VRTS::Enclosure::Discover::AUTHENTICATION_FAILED;
    }
    elsif ($out->[1] =~ /security role privilege violation/i)
    {
        return undef, &VRTS::Enclosure::Discover::SECURITY_ROLE_NOT_PRIVILEGE;
    }
    elsif ($out->[1] =~ /Command is not supported/i or $out->[1] =~ /Feature software is not installed/i)
    {
        # naviseccli is not supported on this array
        return undef, &VRTS::Enclosure::Discover::VENDOR_COMP_VERSION_INCOMPAT;
    }
    elsif (($out->[1] =~ /network error occurred/i) or ($out->[1] =~ /Error occurred/i) or
                ($out->[1] =~ /Could not connect/i) or ($out->[1] =~ /Abort/i)
                or ($out->[1] =~ /Management Server is not running/i) or 
                ($out->[1] =~ /connection refused/i) ) 
    {
        # network error occured while communicating with the array
        # port may not be accessible 
        return undef, &VRTS::Enclosure::Discover::ARRAY_AUTHORIZATION_FAILED;
    }
    
    my %enc_map;
    my @cliop = split('\n', $out->[1]);
    foreach my $data (@cliop)
    {
        if($data =~ /^Agent Rev/){
           $data =~ m/Agent Rev:\s*([0-9]+\.[0-9]+)\s*/;
           my $fw_version = $1;
           if($fw_version < "6.26") {
               return undef,
                      &VRTS::Enclosure::Discover::VENDOR_COMP_VERSION_INCOMPAT;
           }
        }
        if($data =~ /^Model:\s*(.*)/) {
            my $model = $1;
            $model =~ s/^\s*//;
            $model =~ s/\s*$//;
            $enc_map{'MODEL'} = $model; 
        }
        if($data =~ /^Serial No:\s*(.*)/) {
            my $serial = $1;
            $serial =~ s/\s*$//;
            $serial =~ s/^\s*//;
            $enc_map{'SERIAL'} = $serial;
         }
    }
    if (not defined $secfilepath) 
    {
        $arg1 = "-h $server -Port $port -User $params->{'user'}";
        $arg2 = "-Scope $scope arrayname";
    }
    else 
    {
        $secfilepath =~ s/^\s*//;
        $secfilepath =~ s/\s*$//;
        $secfilepath =~ s/\\/\\\\/g;
 
        $arg1 = " -h $server -secfilepath $secfilepath arrayname";
        $arg2 = undef;
    }
    
    my $script = "$vardir/navicli.pl"; 
    _create_probe_script($arg1, $id, $pass, $arg2, $script);

    my $out = VRTS::Enclosure::Utils::push_exec($ch, $script, "\$VARDIR\/navicli.pl", $probe_timeout);

    VRTS::Logger::log('trace', 'COMMAND = [arrayname]'); 
    VRTS::Logger::log('trace', "OUTPUT = [$out->[1]]"); 

    if ($out->[0] != 0)
    {
        my $err = $out->[0] >> 8;
        if ($err == 192)
        {
            print "Request timed out due to no activity for timeout period\n";
            return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
        }
         print "Error in push_exec\n";
         return undef, &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }
    unlink($script); 
    @cliop = split('\n', $out->[1]);
    foreach my $data (@cliop)
    {
        if($data =~ /^Array Name:\s*(.*)/) {
            my $name = $1;
            $name =~ s/^\s*//;
            $name =~ s/\s*$//;
            $enc_map{'NAME'} = $name;
        }
    }

    return \%enc_map; 
}

# Now fill info about the discovered enclosures
sub fill_enc_details
{

    my $response = shift;
    my $cliop = shift;
    my $server = shift;
    my ($serial, $vdid, $model, $product);
	# This is File only
    if(defined $response and not defined $cliop) {
        $vdid = "EMC-Celerra-". $response->{CelerraSystem}->{wwCid};
        $serial = $response->{CelerraSystem}->{serial};
        $model = $response->{CelerraSystem}->{productName};
        $product = $response->{CelerraSystem}->{productName};
    }
	# This is block only
	if(defined $cliop and not defined $response) {
		$vdid = "EMC_CLARiiON_".$cliop->{'SERIAL'};
		$serial = $cliop->{'SERIAL'};
		$model = $cliop->{'MODEL'};
        $product = $cliop->{'MODEL'};

	}

	# Its unified device
	
    if(defined $cliop and defined $response) {
        $vdid = $response->{CelerraSystem}->{wwCid} ."_" . $cliop->{'SERIAL'};
        $serial = $response->{CelerraSystem}->{serial} . "/" .$cliop->{'SERIAL'};
		$model = $response->{CelerraSystem}->{productName};
		$product = $response->{CelerraSystem}->{productName};
    }
    my %val_hash;
	#my $vdid = $vdid;
    $val_hash{VDID} = $vdid;
    $val_hash{product} = $product;
    $val_hash{vendor} = "EMC";
    $val_hash{displayname} = $product . "@" . $server;
	if( not defined ($server)) {
		$val_hash{displayname} = $vdid;
	}
    $val_hash{serial} = $serial;
    $val_hash{model} = $model;
    $enchash->{$vdid} = \%val_hash;
}

# Helper function to execute XML API and return info for VNX file.
sub _get_array_capabilities
{
    my $ch = shift;
    my $id = shift;
    my $server = shift;
    my $user = shift;
    my $pass = shift;

    my $script = File::Spec->catfile($vardir, "celerra.pl.$$");
    my $url = "https://$server:443/Login";
    my $login_header = "cel_login";

    create_script($url, "xmlapi.inp", $script, $login_header, 0, $id, $server, $user, $pass);
    my $out = VRTS::Enclosure::Utils::push_exec($ch, $script, "\$VARDIR/celerra.pl", $probe_timeout);
    VRTS::Logger::log('debug', "Logging login info [$out->[1]]");
    unlink($script);

    VRTS::Logger::log('trace', "COMMAND = [$url]"); 
    VRTS::Logger::log('trace', "OUTPUT = [$out->[1]]"); 

    if ($out->[0] != 0 ) 
    {
        return $out;
    }
    my @logininfo = $out->[1]; 
    my $token;
    foreach my $line (@logininfo)
    {
        if($line =~ /Set-Cookie: (.*)/) {
            $token = $1;
            chomp($token);
        }
    }
    ### Recieved login token end 

    my $session_payload = File::Spec->catfile($vardir, "xmlapi.inp.$$");
    _create_session_payload($session_payload, $token);
    $out = VRTS::Enclosure::Utils::push($ch, $session_payload,
        "\$VARDIR/xmlapi.inp", $probe_timeout);
    if ($out->[0] != 0)
    {
        unlink($session_payload);
        return $out;
    }
    $script = File::Spec->catfile($vardir, "celerra.pl.$$");
    $url = "https://$server:443/servlets/CelerraManagementServices";
    create_script($url, "xmlapi.inp", $script, $login_header, 0);
    $out = VRTS::Enclosure::Utils::push_exec($ch, $script, "\$VARDIR/celerra.pl", $probe_timeout);
    VRTS::Logger::log('debug', "Logging login session info [$out->[1]]");
    unlink($script);
    unlink($session_payload);

    VRTS::Logger::log('trace', "COMMAND = [$url]"); 
    VRTS::Logger::log('trace', "OUTPUT = [$out->[1]]"); 

    ###### Session established end

    if ($out->[0] != 0)
    {
        return $out;
    }
    my $cookie;

    my @arr = $out->[1];
    foreach my $line (@arr) 
    {
        if($line =~ /Set-Cookie: JSESSIONID=(.*)?\;/)
        {
            $cookie = $1;
            $cookie =~ /(.*)\;/;
            $cookie = $1;
        }
    }
    my $probe_payload = File::Spec->catfile($vardir, "xmlapi.inp.$$");
    _create_probe_payload($probe_payload, $token, $cookie);
    $out = VRTS::Enclosure::Utils::push($ch, $probe_payload,
        "\$VARDIR/xmlapi.inp", $probe_timeout);

    VRTS::Logger::log('trace', "COMMAND = [$url]"); 
    VRTS::Logger::log('trace', "OUTPUT = [$out->[1]]"); 

    if ($out->[0] != 0)
    {
        unlink($probe_payload);
        return $out;
    }

    my $script = File::Spec->catfile($vardir, "celerra.pl.$$");
    $url = "https://$server:443/servlets/CelerraManagementServices";
    create_script($url, "xmlapi.inp", $script, $login_header, 1);
    $out = VRTS::Enclosure::Utils::push_exec($ch, $script, "\$VARDIR/celerra.pl", $probe_timeout);
    VRTS::Logger::log('debug', "Logging VNX info [$out->[1]]");
    unlink($probe_payload);
    unlink($script);

    if ($out->[0] != 0)
    {
        return $out;
    }

    my $out1 = _logout($token, $cookie, $server, $ch);
    if (defined($out1) && ($out1->[0] != 0))
    {
        return $out1;
    }
    return $out;

}

sub _logout
{
    my $token =  shift;
    my $cookie = shift;
    my $server = shift;
    my $ch = shift;

    my $logout_payload = File::Spec->catfile($vardir, "xmlapi.inp.$$");

    my $logout_header = "cel_logout";
    _create_logout_payload($logout_payload, $token, $cookie);
   my $out = VRTS::Enclosure::Utils::push($ch, $logout_payload,
       "\$VARDIR/xmlapi.inp", $probe_timeout);
   if ($out->[0] != 0)
   {
	   unlink($logout_payload);
	   return $out;
   }
   my $script = File::Spec->catfile($vardir, "celerra.pl.$$");
   my $url = "https://$server:443/servlets/CelerraManagementServices";
   create_script($url, "xmlapi.inp", $script, $logout_header, 0);
   $out = VRTS::Enclosure::Utils::push_exec($ch, $script, "\$VARDIR/celerra.pl", $probe_timeout);

   unlink($logout_payload);
   unlink($script);

   if ($out->[0] != 0)
   {
	return $out;
   }
   return undef;
}

sub create_script
{
    my ($url, $payload, $script, $login_header, $ignore_header, 
        $id, $server, $user, $pass) = @_;

    open(S, ">$script");
    binmode S;
    print S <<EOF;
#!/opt/VRTSsfmh/bin/perl
BEGIN { \@INC = ("/opt/VRTSsfmh/lib/modules") if ( \$^O !~ /Win32/i ) }
require VRTS::Paths;
require VRTS::Util;
require File::Spec;
my \$xprtlc = VRTS::Paths::get_path('xprtlc');
my \$vardir = VRTS::Paths::get_path('AppDir');
my \$payload = File::Spec->catfile(\$vardir, "$payload");
my (\$id, \$password, \$rc) = ('$id', '$pass');
if (defined(\$id) && (\$id ne '')) {
    (\$rc, \$password) = VRTS\:\:Util\:\:decrypt_passwd(\$password, \$id);
    if (\$rc) {
        print "decrypt_passwd failed with error code = [\$rc]";
        exit \$rc;
    }
    open (C, ">\$payload");
    print C <<EOF1;
POST http://$server/Login HTTP/1.1
Content-Type:application/x-www-form-urlencoded
Host: $server

user=$user&password=\$password&Login=Login
EOF1
    close(C);

}
my \$login_header = File::Spec->catfile(\$vardir, "$login_header");
my \$cmd = VRTS::Util::make_command(\$xprtlc, '-j', \$payload, '-l', "$url", '-D', \$login_header);
system(\$cmd);
if($ignore_header eq 0) {
open(my \$handle, "\$login_header");
while(<\$handle>) {
    print \$_;
}
close \$handle;
}
unlink(\$payload);
EOF
    close(S);
}


sub _create_probe_payload
{
    my $payload_file = shift;
    my $token =shift;
    my $connector_sess = shift;

    open (C, ">$payload_file");
    print C <<EOF;
Content-Type: text/xml
Cookie:$token
Cookie: $connector_sess\; Path=/; Secure
CelerraConnector-Sess: $connector_sess

<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<RequestPacket xmlns="http://www.emc.com/schemas/celerra/xml_api">
<Request>
<Query>
<CelerraSystemQueryParams/>
</Query>
</Request>
</RequestPacket>
EOF
    close(C);
}

#Deprecated
sub _create_login_payload
{
    my $payload_file = shift;
    my $server = shift;
    my $user = shift;
    my $password = shift;
    
    open (C, ">$payload_file");
    print C <<EOF;
POST http://$server/Login HTTP/1.1
Content-Type:application/x-www-form-urlencoded
Host: $server

user=$user&password=$password&Login=Login
EOF
    close(C);
}

sub _create_session_payload
{
    my $payload_file = shift;
    my $token = shift;

    open (C, ">$payload_file");
    print C <<EOF;
Content-Type:text/xml
Cookie: $token

<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<RequestPacket xmlns="http://www.emc.com/schemas/celerra/xml_api"/>
EOF
    close(C);
}

sub _create_logout_payload
{
    my $payload_file = shift;
    my $server = shift;
    my $token = shift;
    my $connector_sess = shift;

    open (C, ">$payload_file");
    print C <<EOF;
POST http://$server/servlets/CelerraManagementServices HTTP/1.1
Content-Type:Content-Type:text/xml
Host: $server

Cookie:$token
$connector_sess
CelerraConnector-Sess:$connector_sess
CelerraConnector-Ctl: DISCONNECT
EOF
    close(C);
}

sub _create_probe_script
{
    my ($arg1, $id, $pass, $arg2, $script) = @_;
    my $unix_cli = "$cli_path/naviseccli";
    my $win_cli = "$cli_path/NaviSECCli.exe";

    $win_cli =~ s/^\s*//;
    $win_cli =~ s/\s*$//;
    $win_cli =~ s/\\/\\\\/g;

    $unix_cli =~ s/^\s*//;
    $unix_cli =~ s/\s*$//;
    $unix_cli =~ s/\\/\\\\/g;

    open(my $S, ">$script");
    binmode $S;
    print $S "\#\!\/opt\/VRTSsfmh\/bin\/perl\n";
    print $S "BEGIN { \@INC = (\"/opt/VRTSsfmh/lib/modules\") if ( \$\^O !~ /Win32/i ) }\n\n";
    if (defined($arg2) && ($arg2 ne '')) {
        print $S "require VRTS\:\:Util;\n\n";
        print $S "my (\$rc, \$pass) = VRTS\:\:Util\:\:decrypt_passwd(\"$pass\", \"$id\");\n";
        print $S "if (\$rc) {\n";
        print $S "print \"decrypt_passwd failed with error code = [\$rc]\";\n";
        print $S "exit \$rc;\n}\n";
    }

    print $S "if ( \$\^O !~ /Win32/i ) {\n";
    print $S "print \"not found\" if (\! -x \"$unix_cli\")\;\n";
    if ((not defined($arg2)) or ($arg2 eq '')) {
        print $S "\t system(\"$unix_cli $arg1\")\;\n}\n";
    } else {
        print $S "\t system(\"$unix_cli $arg1 -Password \$pass $arg2\")\;\n}\n";
    }
    print $S "else {\n";
    print $S "print \"not found\" if (\! -x \"$win_cli\")\;\n";
    if ((not defined($arg2)) or ($arg2 eq '')) {
        print $S "\t system(\" \\\"$win_cli\\\" $arg1\")\;\n}\n";
    } else {
        print $S "\t system(\" \\\"$win_cli\\\" $arg1 -Password \$pass $arg2\")\;\n}\n";
    }
    close($S);
}

sub provide_capabilities
{
    my $self = shift;
    return \%capabilities;
}

1;
