#$Id$
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2016 Veritas Technologies LLC.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# VERITAS TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
#
# The  Licensed  Software  and  Documentation  are  deemed  to be commercial
# computer  software  as  defined  in  FAR  12.212 and subject to restricted
# rights  as  defined in FAR Section 52.227-19 "Commercial Computer Software
# - Restricted  Rights"  and  DFARS 227.7202,  et seq.  "Commercial Computer
# Software  and  Commercial Computer Software Documentation," as applicable,
# and any successor regulations, whether delivered by Veritas as on premises
# or  hosted  services.  Any  use,  modification,  reproduction  release,
# performance,  display  or  disclosure  of  the  Licensed  Software  and
# Documentation by the U.S. Government shall be solely in accordance with
# the terms of this Agreement. $

package VRTS::Enclosure::Discover::XIV;
use strict;
use Carp;
require Exporter;
require VRTS::Paths;
require VRTS::Enclosure::Utils;
require VRTS::Enclosure::VDID;

my %capabilities = ("VOL_CAPABLE" => 0, "SHOW_RG_LDEV" => 1, "SHOW_RG_PDEV" => 0, "SHOW_PDEV" => 1, "SHOW_LDEV" => 1,
                     "SHOW_REPLICATION" => 1, "SHOW_TP_SOURCELDEV" => 0, "SHOW_TP_THINLDEV" => 1, "SHOW_LDEV_PDEV" => 0, "LDEV_PDEV_MAP" => 0,
                     "PRIMARY_REMOTE_REP_CAPABLE" => 1, "SHOW_VSET" => 0, "SHOW_TP" => 1, "SHOW_RG" => 1, "SECONDARY_REMOTE_REP_CAPABLE" => 0,
                     "SHOW_PDEV_FOR_THIN_LDEV" => 0, "SHOW_PDEV_FOR_TP_SOURCE_THINLDEV" => 0,
                     "SHOW_DEVICE_GROUP_REPLICATION" => 0, "STATS_CAPABLE" => 1);

my $vardir;
my $script;
my $dscript;
$vardir= VRTS::Paths::get_path("AppDir");
$script = "$vardir/xivscript$$.pl";
$dscript = "$vardir/dxivscript$$.pl";

sub new 
{
    my $class = shift;
    my $self = {};

    bless ($self, $class);
    return $self;
}

sub initialize
{
    my $top = shift;
    my $module = new VRTS::Enclosure::Discover::XIV;
    return $module;

}

sub test_exec_xcli
{
    my ($ch, $server, $user, $id, $pass, $path, $cmd) = @_;
    my $rc = 0;

    _create_script_for_command($server,$user, $id, $pass,$path,'config_get',$script,'csv');
    my $out = VRTS::Enclosure::Utils::push_exec($ch, $script, "\$VARDIR\/xivscript.pl");
    if ($out->[0] != 0 or $out->[1] =~ /not found/i)
    {
         unlink($script);
         return 1;
    }
    unlink($script);
    return 0;
}

sub exec_xcli
{
    my ($ch, $server, $user, $id, $pass, $path, $cmd, $format) = @_;
    my $rc = 0;

    _create_script_for_command($server,$user, $id, $pass,$path,$cmd,$script,$format);
    my $out = VRTS::Enclosure::Utils::push_exec($ch, $script, "\$VARDIR\/xivscript$$.pl");
    if ($out->[0] != 0  or $out->[1] =~ /not found/i)
    {
         exit 0;
    }
    unlink($script);
    return $out;
}
sub get_enclosure_listing
{
    my $self = shift;
    my $params = shift;

    my $ch = $params->{"control-host"};
    my $id = $params->{ID};
    my $server = $params->{"server"};
    my $server2 = $params->{ARRAY_XIV_IP2};
    my $server3 = $params->{ARRAY_XIV_IP3};
    if (defined($server2) && ($server2 ne '')) {
        VRTS::Logger::log('debug', "server2 = $server2");
        $server .= ',' . $server2; 
    }
    if (defined($server3) && ($server3 ne '')) {
        VRTS::Logger::log('debug', "server3 = $server3");
        $server .= ',' . $server3; 
    }
    my $user = $params->{"user"};
    my $pass = $params->{"pass"};
    my $path = $params->{"install-path"};
    my $probe_timeout = $self->{"probe_timeout"};

    VRTS::Logger::log('trace', "control-host = [$ch]"); 
    VRTS::Logger::log('trace', "ID = [$id]"); 
    VRTS::Logger::log('trace', "server = [$server]"); 
    VRTS::Logger::log('trace', "server2 = [$server2]"); 
    VRTS::Logger::log('trace', "server3 = [$server3]"); 
    VRTS::Logger::log('trace', "path = [$path]"); 
    VRTS::Logger::log('trace', "user = [$user]"); 

    my $rc = 0;

    _create_script_for_command($server,$user, $id, $pass,$path,"config_get",$script);
    my $out = VRTS::Enclosure::Utils::push_exec($ch, $script, "\$VARDIR\/xivscript.pl", $probe_timeout);

    VRTS::Logger::log('trace', 'COMMAND = [config_get]'); 
    VRTS::Logger::log('trace', "OUTPUT = [$out->[1]]"); 

    if ($out->[0] != 0)
    {
        my $err = $out->[0] >> 8;
        if ($err == 192)
        {
            return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
        }
         return undef,&VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }
    elsif ($out->[1] =~ /not found/i || $out->[1] =~ /No such file/i)
    {
         return undef,&VRTS::Enclosure::Discover::VENDOR_COMP_NOT_FOUND;
    }
    elsif ($out->[1] =~ /Incorrect password/i)
    {
         return undef,&VRTS::Enclosure::Discover::INCORRECT_PASSWORD;
    }
    elsif ($out->[1] =~ /USER_NAME_DOES_NOT_EXIST/i)
    {
         return undef,&VRTS::Enclosure::Discover::USER_NAME_DOES_NOT_EXIST;
    }
    elsif ($out->[1] =~ /NO_CONNECTION/i)
    {
         return undef,&VRTS::Enclosure::Discover::NO_CONNECTION;
    }
    elsif ($out->[1] =~ /LOGIN_FAILURE_USER_FAILED_TO_LOGIN/i)
    {
        return undef, &VRTS::Enclosure::Discover::AUTHENTICATION_FAILED;
    }

    unlink($script);

    my %data1;
    my $objid;
    my $enchash;
    if ($out->[0] == 0)
    {
        my @data = split ("\n",$out->[1]);
        foreach my $d (@data)
        {
            $d =~ s/\"//g;
            my @arr = split /,/,$d;
            $data1{'displayname'} = $arr[1]  if ($arr[0] eq "system_name");
            $data1{'serial'} = $arr[1]  if ($arr[0] eq "system_id");
            $data1{'model'} = $arr[1]  if ($arr[0] eq "machine_model");
            $data1{'type'} = $arr[1]  if ($arr[0] eq "machine_type");
            $data1{'vendor'} = "IBM";
            $data1{'product'} = "IBM XIV";
            $data1{'VDID'} = $arr[1]  if ($arr[0] eq "system_name");
            $objid = $arr[1]  if ($arr[0] eq "system_name");
        }

        my $vdid_obj = new VRTS::Enclosure::VDID ();
        $vdid_obj->initialize("XIV", 0);
        my ($vdid, $oldvdid)  = 
                $vdid_obj->get_enclosure_vdid("IBM",$data1{'type'},$data1{'serial'});

        if (defined $oldvdid and $oldvdid ne $vdid)
        {
            $data1{'OLDVDID'} = $oldvdid;
        }
        $data1{'VDID'} = $vdid;
        $enchash->{$vdid} = \%data1;
    }

    return $enchash;
}

sub _create_script_for_command
{
    my ($server, $user, $id, $pass, $path, $cmd, $script, $format) = @_;
    my $iplist = '';
    my $cli;
    my $args;

    $path =~ s/\\/\\\\/g;
    $cmd =~ s/^\s*//;
    $cmd =~ s/\s*$//;
    $cli = $path . "/xcli";

    if($server =~ /,/)
    {
        my @ips = split(/,/, $server);
        foreach(@ips)
        {
            $iplist .= " -m $_";
        }
    }
    else
    {
        $iplist = "-m $server";
    }

    if($format eq 'xml')
    {
        $args = " -x -y -u $user";
    }
    else
    {
        $args = " -s -y -u $user";
    }

    my $r = open(my $fh, '>', $script);
    if (not defined($fh)) {
        VRTS::Logger::log('error', "Unable to open file $script for writing");
        exit $r;
    }

    binmode $fh;
    print $fh <<EOF;
#!/opt/VRTSsfmh/bin/perl
BEGIN { \@INC = ('/opt/VRTSsfmh/lib/modules') if ( \$^O !~ /Win32/i ) }
require VRTS::Util;
my (\$rc, \$pass) = VRTS::Util::decrypt_encoded_passwd('$pass', '$id');
if (\$rc) {
    print "decrypt_encoded_passwd failed with error code = [\$rc]";
    exit \$rc;
}   
\$pass = VRTS::Util::esc_special_chars(\$pass);
if ( \$^O !~ /Win32/i ) {
    my \$cli = "$cli";
    if (! -x \$cli) {
        print "not found";
        return;
    }
    eval {
        VRTS::Util::system_with_timeout(1800, "\$cli $args -p \$pass $iplist $cmd");
	alarm(0);
    };
    if(\$@) {
        print "system_with_timeout failed with Error=\$@";
	return;
    }
} else {
    my \$cli = "$cli.exe";
    if (! -x \$cli) {
        print "not found";
        return;
    }
    eval {
        VRTS::Util::system_with_timeout(1800, "\\\"\$cli\\\" $args -p \$pass $iplist $cmd");
	alarm(0);
    };
    if(\$@) {
        print "system_with_timeout failed with Error=\$@";
	return;
    }
}
EOF
    close($fh);

}

sub provide_capabilities
{
    my $self = shift;

    return \%capabilities;
}

1;
