#$Id$
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2016 Veritas Technologies LLC.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# VERITAS TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
#
# The  Licensed  Software  and  Documentation  are  deemed  to be commercial
# computer  software  as  defined  in  FAR  12.212 and subject to restricted
# rights  as  defined in FAR Section 52.227-19 "Commercial Computer Software
# - Restricted  Rights"  and  DFARS 227.7202,  et seq.  "Commercial Computer
# Software  and  Commercial Computer Software Documentation," as applicable,
# and any successor regulations, whether delivered by Veritas as on premises
# or  hosted  services.  Any  use,  modification,  reproduction  release,
# performance,  display  or  disclosure  of  the  Licensed  Software  and
# Documentation by the U.S. Government shall be solely in accordance with
# the terms of this Agreement. $
#$Source$

package VRTS::Enclosure::VDID::HPEVA;
use strict;
use Carp;
require Exporter;


sub new 
{
    my $class = shift;
    my $self = {};

    bless ($self, $class);
    return $self;
}

sub initialize
{
    my $top = shift;

    my $module = new VRTS::Enclosure::VDID::HPEVA;
    return $module;

}

# get_enclosure_vdid - Returns the Enclosure VDID
# Accepts as input the Vendor Name (COMPAQ), product
# (HSV) and the enclosure wwn
# If any of the values is missing, then an undef
# value is returned
#
# Example: COMPAQ_HSV_5000-1FE1-500A-8F00
#
sub get_enclosure_vdid
{
    my ($self, $vendor, $product, $arraywwn) = @_;

    return $vendor .'_' . $product . '_' . $arraywwn if ($vendor and $product and
            $arraywwn);
    return undef;
}

# get_disk_vdid - Returns the vdid for the
# array Lun (LDEV).
# Accepts as input the Vendor (COMPAQ), the
# product (HSV), the enclosure wwn and the
# wwlunid
# If any of the values is missing, then an undef
# value is returned
#
# Example: COMPAQ_HSV_5000-1FE1-500A-8F00_6005-08B4-0005-440B-0001-6000-0379-0000
#
sub get_disk_vdid
{
    my ($self, $vendor, $product, $arraywwn, $wwlunid) = @_;

# Change the wwlunid of the vdisk to
# upper-case. 
# wwlunid is like this 0500-01fe-0102-3bc0-0500-01fe-0102-3bc1
# Change this to 0500-01FE-0102-3BC0-0500-01FE-0102-3BC1
    $wwlunid = uc($wwlunid);
    $arraywwn = uc($arraywwn);
    # We want to generate an ldev vdid.
    return $vendor . '_' . $product . '_' . $arraywwn . '_' . $wwlunid 
        if ($vendor and $product and $arraywwn and $wwlunid);
    return undef;
}

1;

