#$Id$
# VT50-2000-2016-84-28-4
# $Copyright (c) 2017 Veritas Technologies LLC.  All rights reserved. Veritas
# and the Veritas Logo are trademarks or registered trademarks of Veritas
# Technologies LLC or its affiliates in the U.S. and other countries. Other 
# names may be trademarks of their respective owners.
#
# The Licensed Software and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software - Restricted Rights" and
# DFARS 227.7202, et seq. "Commercial Computer Software and Commercial Computer
# Software Documentation," as applicable, and any successor regulations, whether
# delivered by Veritas as on premises or hosted services. Any use, modification,
# reproduction release, performance, display or disclosure of the Licensed Software
# and Documentation by the U.S. Government shall be solely in accordance with the
# terms of this Agreement. $
package jobs::probe_array;

require VRTS::Paths;
my $bdir = VRTS::Paths::get_path("InstallDir");
$bdir=$bdir."/agentlet/";
push @INC, $bdir;

use VRTS::OS;
use VRTS::CMD;
use VRTS::CfgFile;
use File::Spec;
use Sys::Hostname;
use threads;
use threads::shared;

use JSON;
use strict;
require Exporter;
use vars qw($VERSION @ISA @EXPORT );
use VRTS::CGILOCALE;

require XML::Simple;
use Data::Dumper;
require VRTS::Enclosure::Discover;
require VRTS::Enclosure::VDID;

@ISA = qw(Exporter);

$VERSION = '1.00';

@EXPORT = qw(initialize execute checkenv);

# Fill this information in data structure of this Job
my $job=undef;
my $in_params;

sub initialize
{
    my $agent = shift;
    my $index = shift;

    # Get params passed to this operation
    $in_params = $agent->get_param();

    # Get a handle to this Job created during driver initialization
    $job = $agent->get_job_by_name("probe_array");
    return 1 if(not defined $job);

    return 0;
}

sub execute
{
    my $agent = shift;
    my $finalrc=0;

    my $method = $in_params->{method};

    $method =~ s/\s*//g;
    return 1 if (not defined $method or $method eq "");

    my $enc_info = new VRTS::Enclosure::Discover();
    $enc_info->initialize("$method"); 

    my $threxit:shared = 0;

    # Start a thread that will print '\n' every 4 mins to keep the
    # conenction alive for more than 5 mins if array probe doesn't
    # return in that time
    my $to_thr = threads->new( sub { 
            lock($threxit); # This is a blocking call
            my $wait_time_sec = 240; # Max time to wait: 240 sec
            while ($threxit == 0) {
                # cond_timedwait takes a locked variable and abs time for
                # timeout. It comes out because of timeout or because it was
                # signalled. In either case, it will relock the variable before
                # returning
                if (!cond_timedwait($threxit, time() + $wait_time_sec)) {
                    # Timeout happened
                    print "\n";
                } else {
                    # We were signalled. The loop will exit if $threxit != 0
                }
            }
            });

    #get_enclosure_listing will go ahead with executing probe script
    #with given parameters
    my ($out, $ret_code, $err_msg) = $enc_info->get_enclosure_listing($in_params);
    #get_enclosure_listing has now returned and we will now signal the other 
    #printing thread so that it can exit from the waiting loop 
    if (defined $to_thr)
    {
        {
            # lock() is a blocking call. But the printing thread has either not
            # started or should be in cond_timedwait at which time we'll get the
            # lock
            lock($threxit);
            $threxit = 1;
            # If nobody is cond_wait'ing cond_signal is a no-op, which is ok
            cond_signal($threxit);
        } # End of scope for lock causes it to be unlocked. There is no unlock call

        # Wait for printing thread to finish
        $to_thr->join();
    }


    if (defined $ret_code and $ret_code != &VRTS::Enclosure::Discover::PROBE_SUCCESS)
    {
        $job->set_error($err_msg);
        return $ret_code;
    }

    my $js = JSON->new->allow_nonref;
    my $str = $js->encode($out);
    $job->set_output($str);
    $agent->set_output($str);

    return 0;
}

sub checkenv
{
    my $agent = shift;
    return 0;
}

sub rollback
{
    my $agent = shift;
    return 0;
}

1;

