#!/opt/VRTSsfmh/bin/perl
#$Id$
# VT50-2000-2016-84-28-4
# $Copyright (c) 2017 Veritas Technologies LLC.  All rights reserved. Veritas
# and the Veritas Logo are trademarks or registered trademarks of Veritas
# Technologies LLC or its affiliates in the U.S. and other countries. Other 
# names may be trademarks of their respective owners.
#
# The Licensed Software and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software - Restricted Rights" and
# DFARS 227.7202, et seq. "Commercial Computer Software and Commercial Computer
# Software Documentation," as applicable, and any successor regulations, whether
# delivered by Veritas as on premises or hosted services. Any use, modification,
# reproduction release, performance, display or disclosure of the Licensed Software
# and Documentation by the U.S. Government shall be solely in accordance with the
# terms of this Agreement. $

BEGIN { @INC = ("/opt/VRTSsfmh/lib/modules") if ($^O !~ /Win32/i);}

use strict;
use VRTS::Paths;
use File::Spec;
use VRTS::Util;
use VRTS::Ini;
use File::Temp;
use JSON;
use VRTS::Logger;
use VRTS::Schedule;

my $context = $ENV{context};
if (!defined $context)
{
    $context = "SI";
}

my $installdir = VRTS::Paths::get_path ("InstallDir");
my $app_dir = VRTS::Paths::get_path ("AppDir");
my $appdircs = $app_dir;
   $appdircs =~ s/VRTSsfmh/VRTSsfmcs/;
my $app_dir = VRTS::Paths::get_path ("AppDir");
my $addon_dir = $ENV{Store};

my $perl = File::Spec->catfile ($installdir, "bin", "perl");
my $array_driver = File::Spec->catfile ($installdir, "bin", "array_driver.pl");

my $log_dir = File::Spec->catdir($app_dir,"logs");
my $log = new VRTS::Logger(Module => 'siplugin',
                           Logdir => $log_dir,
                           Level => 'debug',
                           Logfile => 'siplugin-setup.log',
                           UmiString => '');

if ($context eq "sdktest")
{
   exit (0);
}

my $plugin_configs;
my $vendor;
my $model;
my $array_family;
my $version;
my $addon_id;
my $ha;
my $active_cms = 0;
my $config_data;

my $ret = get_env();
goto err if ($ret);

VRTS::Logger::log ('debug', "Enable called for Plug-in $array_family");

 

if ($active_cms) 
{
   if ($ha)
     {
         # Share the app directory for the plug-in
         my %fam_hash = ();
         my $family_app_dir = File::Spec->catdir(VRTS::Paths::get_path("AppDir"), $array_family);
         $fam_hash{$family_app_dir}->{dir} = 1;
         $fam_hash{$family_app_dir}->{perm} = 755;

         &share_files_for_ha(\%fam_hash, 1); 
     }
     foreach my $encl_config (@$plugin_configs)
     {
         my $enc_id = $encl_config->{VENDOR_ID};
         my $schedid = $enc_id;

         if (defined $enc_id && $enc_id ne "")
         {
             VRTS::Logger::log ('debug', "enabled schedule for $enc_id");
             my $array_cmd = VRTS::Util::make_command($perl, $array_driver, "--family", $array_family, 
                             "--hidden", "--context", "vardir=$enc_id");
             VRTS::Schedule::set_sched($schedid, ":", "360", $array_cmd);

             # Resume if family was already scheduled and paused
             VRTS::Schedule::pause_resume_sched($schedid, "resume");
         }
     }
}
else
{
   # This is CMS HA setup and enable is getting called on the standby node.
   # Nothing to enable on slave.
}

VRTS::Logger::log('debug' , "Successfully enabled SI plug-in for $array_family($version)\n");
print "Successfully enabled SI plugin for $array_family($version)\n";
exit(0);

err:
    VRTS::Logger::log('error' , "Failed to enable SI Plugin for $array_family($version)\n");
    print "Failed to enable SI Plugin for $array_family($version)\n";
    exit(1);


sub get_env
{
    my $msg;

    VRTS::Logger::log ('debug', "addon-dir is $addon_dir");
    if ($context ne "sdktest")
    {
        my $cms_info = VRTS::Util::get_cms_config_info();
        if ($cms_info->{'cs'} == 0)
        {
            $msg = "Current host is not a Management Server.\n".
                   "SI Plug-in should be installed only on a Management Server\n";
            print STDERR $msg;
            VRTS::Logger::log ('error', $msg);
            return 1;
        }

        $ha = $cms_info->{'ha'};
        if ($ha)
        {
            if ($cms_info->{'active_node'} == 1)
            {
               # this is a CMS HA active node
               $active_cms = 1;
            }
        }
        else
        {
            $active_cms = 1;
        }
        VRTS::Logger::log('debug', "active_cms = $active_cms. ha = $ha");
    }

    my $config_file = File::Spec->catfile ($addon_dir, "plugin_def.conf");
    if (open (H, $config_file) == 0)
    {
        $msg = "failed to open file $config_file : $@";
        VRTS::Logger::log ('error', $msg);
    }
    else
    {
        my $buf = do { local $/; <H> };
        close (H);
        $config_data;
        eval { $config_data = JSON::from_json($buf)};
        if ($@)
        {
            $msg = "Invalid config file $config_file : $@\n";
            VRTS::Logger::log ('error', $msg);
            return 1;
        }
        else
        {
           $vendor = $config_data->{VENDOR};
           $model = $config_data->{MODEL};
           $array_family = $config_data->{ARRAY_PLUGIN_FAMILY};
           $version = $config_data->{ARRAY_PLUGIN_VERSION};
           $addon_id = $config_data->{PLUGIN_ADDON_ID};

           if (!defined $vendor || $vendor eq ""
               || !defined $model || $model eq ""
               || !defined $array_family || $array_family eq ""
               || !defined $version || $version eq "")
           {
               VRTS::Logger::log('error', "Invalid plugin config file. Missing vital information");
               return 1;
           }
           $version =~ s/^v//;

           VRTS::Logger::log('debug', "Vendor [$vendor].Model [$model].".
                   "Family [$array_family].Plugin vesion [$version].Addonid $addon_id");
        }
    }

    my $inc_agentlet = File::Spec->catfile($installdir, "agentlet");
    push @INC, $inc_agentlet;

    if(eval "require VRTS::Enclosure::SIPlugin::Util")
    {
        if ($active_cms)
        {
            $plugin_configs = &{\&{"VRTS::Enclosure::SIPlugin::Util::get_all_plugin_configs"}}($array_family);
        }
    }
    else
    {
        VRTS::Logger::log('error', "Failed to load module:$@.");
        print STDERR "Plug-in not installed completely\n".
                     "Please re-install the plug-in\n";
        return 1;
    }
    return 0;
}

# Utility method to share files/directories between VOM CMS HA nodes
# currently implemented to share directories only.
# shared directory will be created on the HA mountpoint which was configured when CMS HA was setup.
# if the source directory already exists then its data is copied over to the shared directory only in case
# the shared directory does not already exist.
sub share_files_for_ha
{
    my $files = shift; # file or dir
    my $active = shift; # 1 is active node, 0 is standby

    return 1 if (!defined $files  or !defined $active or $active eq "");

    my $sfm_app_dir = VRTS::Paths::get_path("AppDir");
    $sfm_app_dir = File::Spec->canonpath($sfm_app_dir);
    my $sfm_cs_app_dir = $sfm_app_dir;
    $sfm_cs_app_dir =~ s/VRTSsfmh/VRTSsfmcs/;

    my $sfm_resolv = VRTS::Paths::get_path("SFMResolv");
    my @domains;
    my $domain;
    my $val;
    my $ha_mntpt;
    my $cmd;
    my $out;
    my $LN = "/bin/ln";

    VRTS::Ini->query_section($sfm_resolv, "domains", \@domains);
    foreach $domain (@domains){
        VRTS::Ini->query_value($sfm_resolv, $domain, "sfm_cs", \$val);
        if (defined $val && $val == 1){
            VRTS::Ini->query_value($sfm_resolv, $domain, "ha_mountpoint", \$ha_mntpt);
            last;
        }
    }
    return 1 if (!defined $ha_mntpt || $ha_mntpt eq ""); # host is not a CMS or is a CMS but not HA configured

    foreach my $f (keys %$files)
    {
        my $dir = $files->{$f}->{dir}; # 1 for dir. 0 for file
        my $perm = $files->{$f}->{perm}; # file/dir permissions
        $dir = 0 if (!defined $dir); # assume file if not specified
        if ($dir)
        {
            $f =  File::Spec->canonpath($f);
            next if ($f eq $sfm_cs_app_dir || $f eq $sfm_app_dir);
            my $shared_dir_loc = $f;

            if ($^O =~ /Win32/i) {
                $shared_dir_loc =~ s/(.+)(VRTSsfmh|VRTSsfmcs)(.+)/$2$3/;
            }
            VRTS::Logger::log ('debug', "Sharing directory $f");

            $shared_dir_loc = File::Spec->catdir($ha_mntpt,$shared_dir_loc);
            if ($active)
            {
                if (! -d $shared_dir_loc)
                {
                   eval {
                        File::Path::mkpath($shared_dir_loc, 0, $perm);
                   };
                   if ($@)
                   {
                       VRTS::Logger::log('error',"Failed to create dir $shared_dir_loc:$@");
                       next;
                   }
                   # move data from non-shared location to the shared location
                   if ( -d $f)
                   {
                       VRTS::Logger::log('error',"Move $f to $shared_dir_loc");
                       eval {
                           if($^O =~ /Win32/i)
                           {
                               copy_recursively($f,$shared_dir_loc);
                               chdir $sfm_app_dir;
                               File::Path::rmtree($f);
                           }
                           else
                           {
                               my $link_target = readlink($f);
                               next if ($shared_dir_loc eq $link_target);
                               File::Copy::move($f, $shared_dir_loc);
                           }
                       };
                   }
                }
            }
            # Re-create the link even if it already exists. we have nothing to loose.
            if ($^O =~ /Win32/i)
            {
                 $cmd = "rmdir \"$f\"";
                `$cmd 2>&1`;
                 $cmd = VRTS::Util::make_command("mklink", "/D", $f, $shared_dir_loc);
            }
            else
            {
                unlink($f);
                $cmd = VRTS::Util::make_command($LN,"-s",$shared_dir_loc, $f);
            }
            $out = `$cmd 2>&1`;
            if ($?)
            {
                VRTS::Logger::log('error',"Failed to link $f to $shared_dir_loc:$out:$?");
                next;
            }
        }
    }
}
