# $Id$
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2016 Veritas Technologies LLC.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# VERITAS TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
#
# The  Licensed  Software  and  Documentation  are  deemed  to be commercial
# computer  software  as  defined  in  FAR  12.212 and subject to restricted
# rights  as  defined in FAR Section 52.227-19 "Commercial Computer Software
# - Restricted  Rights"  and  DFARS 227.7202,  et seq.  "Commercial Computer
# Software  and  Commercial Computer Software Documentation," as applicable,
# and any successor regulations, whether delivered by Veritas as on premises
# or  hosted  services.  Any  use,  modification,  reproduction  release,
# performance,  display  or  disclosure  of  the  Licensed  Software  and
# Documentation by the U.S. Government shall be solely in accordance with
# the terms of this Agreement. $
package VRTS::SISdk::Util;
use strict;
use vars qw($VERSION @ISA @EXPORT);

require Exporter;
require JSON;
require VRTS::CMD;
require VRTS::Ini;
require File::Spec;
require VRTS::Paths;
require VRTS::Event;
require VRTS::Lock;
require VRTS::OS;
use File::Copy;
use File::Temp;
use IO::Select;
use JSON;
use FileHandle;
require VRTS::AddOnInfo;

@ISA = qw(Exporter);
@EXPORT = qw( GetBasename handleFileUpload uploadFile);

$VERSION='1.0';

sub uploadFile{
    my($upfile, $addon_dir_name) = @_;
    my $uploadFileName = GetBasename($upfile);
    my $upload_file =  File::Spec->catdir($addon_dir_name,$uploadFileName);

    if (! open(OUTFILE, ">$upload_file") ) {
        my $msg = "Internal Error";
        VRTS::Logger::log('error',"Failed to open $upload_file for writing:$@");
        return 1, $msg;
    }
    my $nBytes = 0;
    my $totBytes = 0;
    my $buffer = "";

    # If you're on Windows, you'll need this. Otherwise, it has no effect.
    binmode(OUTFILE);
    while ( $nBytes = read($upfile, $buffer, 1024) ) {
        print OUTFILE $buffer;
        $totBytes += $nBytes;
    }
    close(OUTFILE);
    return (0, $uploadFileName);
}

sub GetBasename {
    my $fullname = shift;
    my(@parts);
    if ( $fullname =~ /(\\)/ ) {
        @parts = split(/\\/, $fullname);
    } else {
        @parts = split(/\//, $fullname);
    }
    return(pop(@parts));
}

sub handleFileUpload{
    my $upfile = shift;
    my $addon_dir_name = shift;
    my $installdir = shift;
    my ($ret, $uploadedFileName) = uploadFile($upfile, $addon_dir_name);
    if (-T $uploadedFileName)
    {
            my $dos2unix   = File::Spec->catfile($installdir, "bin", "dostounix.pl");
            my $perl = File::Spec->catfile($installdir, "bin", "perl");
            my $uploadedFileFullPathName =  File::Spec->catdir($addon_dir_name,$uploadedFileName);
            my $cmd = VRTS::Util::make_command($perl, $dos2unix, $uploadedFileFullPathName);
            `$cmd`;
    }
    return ($ret, $uploadedFileName);
}

1;

