#!/opt/VRTSsfmh/bin/perl

BEGIN { @INC = ("/opt/VRTSsfmh/lib/modules") if ( $^O !~ /Win32/i ); }

use strict;
use Getopt::Long;
use JSON;
use VRTS::Paths;
use File::Spec;
use File::Path;
use File::Copy;
use File::Temp;
use VRTS::Util;
use Archive::Tar;
use CGI;
use VRTS::SISdk::Fabricate;
use VRTS::Ini;
use IO;
use Inline::Files;

my $fh = new IO::Handle;
my $sdk_version = "1.0";

my $app_dir = VRTS::Paths::get_path("AppDir");
my $log_dir = File::Spec->catdir($app_dir,"logs");
my $installdir  = VRTS::Paths::get_path ("InstallDir");
my $appdircs = $app_dir;
   $appdircs =~ s/VRTSsfmh/VRTSsfmcs/;
my $perl = File::Spec->catfile($installdir, "bin", "perl");
my $xprtlc = File::Spec->catfile($installdir, "bin", "xprtlc");
my $sfme = File::Spec->catfile($installdir, "bin", "sfme");
my $array_driver = File::Spec->catfile($installdir, "bin", "array_driver.pl");
my $family;
my $vendor;
my $model;
my $discv_method;
my $addon_id;
my $test;
my $test_source;
my $test_source_file;
my $test_source_dir;
my $test_log;
my $config_fname = "plugin_def.conf";
my @discovery_files;

my $fh = new IO::Handle;
$fh->autoflush(1);
$fh->fdopen(fileno(STDOUT),"w") || die "Couldn't open STDOUT: $!";

my $query;
main();

sub main
{
    $query = new CGI();

    my $msg;
    my $head;
    my $body;
    my $tail;
    my $ret = 0;
    my $status = "200 OK";
    my $head = "";

    if ($^O =~ /Win32/i)
    {
        $head = <<'END';
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
"http://www.w3.org/TR/2002/REC-xhtml1-20020801/DTD/xhtml1-strict.dtd">
<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
<html><head><title></title></head><body>
END

        chomp $head;
        $head = "Content-Type: text/html;encoding=utf-8;charset=utf-8\n\n".$head;
        my $whitespace = " " x (1024 - length $head);
        $head =~ s/<body>$/$whitespace<body>/s;
    }
    else
    {
        $head = $query->header ("text/html;encoding=utf-8;charset=utf-8");
    }

    my $whitespace = " " x (1024 - length $head);
    $head =~ s/<body>$/$whitespace<body>/s;
  

  
    my_print($head, 2);
    my_print ($query->start_html(), 2);
 
    my $log = new VRTS::Logger(Module => 'sisdk',
                           Logdir => $log_dir,
                           Level => 'debug',
                           Logfile => 'sisdk.log',
                           UmiString => '');

    my $path_info = $query->path_info();
    if ($path_info =~ /^\/createplugin/)
    {
        ($ret, $msg) = create_plugin($query);
        goto out;
    }
    else
    {
        $ret = 1;
        $msg = "Invalid request parameters\n";
    }

out:

    $body = VRTS::Util::get_cgi_res ($ret, $msg);
    $body .= "\n";
    my_print("OperationResult:$body");

    my_print ($query->end_html());
    exit(0);
}

# Description: Method creates a custom add-on for the SI Plugin
# based on the specified capabilities in the configuration file
# Input :
#       query - instance of CGI object
# Output :
#     retval : 0 on success. nonzero on failure
#     msg : the add-on location on success. Error msg on failure 
sub create_plugin
{
    my $query = shift;
    my $out;
    my $cmd;

    my_print("Fabricating the plugin discovery agentlet...");

    VRTS::Logger::log('debug', "Called create_plugin with config data". $query->param('config-data'));
    my $config_data = $query->param('config-data');
    if ($config_data =~ /^\$VARDIR/)
    {
        $config_data =~ s/\$VARDIR/$app_dir/;
    }
    $config_data = File::Spec->canonpath($config_data);
    VRTS::Logger::log('debug', "create_plugin config data is $config_data");

    # Read the plugin configuration file
    my ($ret, $msg) = read_config($config_data);
    if ($ret)
    {
        return 1, $msg;
    }

    my $data = $msg;
    $test = $query->param('test') if (defined $query->param('test'));
    if ($test)
    {
        $test_source = $query->param('test-source');
        if (!defined $test_source)
        {
            $test_source = 0;
        }

        VRTS::Logger::log('debug', "test option selected with test source $test_source".$query->param('test-source-file'));
        if ($test_source == 1)
        {
            $test_source_file = $query->param('test-source-file');
            if ($test_source_file =~ /^\$VARDIR/)
            {
                $test_source_file =~ s/\$VARDIR/$app_dir/;
            }
            if ( !-f $test_source_file)
            {
                $msg = "discovery command output file $test_source_file not present";
                VRTS::Logger::log('error', $msg);
                return 1, $msg;
            }
            $test_source_dir = File::Basename::dirname($test_source_file);
            chdir $test_source_dir;
            if (VRTS::OS->ungzip_and_untar($test_source_dir, $test_source_file))
            {
                $msg = "Failed to untar the discovery command output files $test_source_file";
                VRTS::Logger::log('error', $msg);
                return 1, $msg;
            }
        }
        $test_log = $query->param('test-log');
        if (!defined $test_log)
        {
            $test_log = File::Spec->catfile ($app_dir, "logs", $family."_Test".$$);
        }
        else
        {
            if ($test_log =~ /^\$VARDIR/)
            {
                $test_log =~ s/\$VARDIR/$app_dir/;
            }
            $test_log = File::Spec->canonpath($test_log);
            VRTS::Logger::log('debug', "create_plugin. test log is $test_log");
        }
    }


    my $workarea = File::Spec->catdir($app_dir, "SISdk", "tmp$$");
    my $tmp_sdkdir = File::Spec->catdir($app_dir, "SISdk"); 

    # Create the staging area for the plugin add-on
    ($ret, $msg) = setup_stage_area($workarea, $data);
    if ($ret)
    {
        chdir $tmp_sdkdir;
        File::Path::rmtree($workarea);
        return 1, $msg;
    }
    my $addon_dir_fqn = $msg;

    # Fabricate the discovery agentlet for the plugin in the stage area
    eval {
        my $plugin_comp = VRTS::SISdk::Fabricate::create_discovery_agentlet($family,$addon_dir_fqn, $data);
    };
    if ($@)
    {
        $msg = "Internal Error";
        VRTS::Logger::log('debug', "Failed to create agentlet:$@");
        chdir $tmp_sdkdir;
        File::Path::rmtree($workarea);
        return 1, $msg;
    }
    # Fabricate the probe module for the plugin in the stage area
    eval {
        my $plugin_comp = VRTS::SISdk::Fabricate::create_probe_module($family,$addon_dir_fqn, $data);
    };
    if ($@)
    {
        $msg = "Internal Error";
        VRTS::Logger::log('debug', "Failed to create probe module:$@");
        chdir $tmp_sdkdir;
        File::Path::rmtree($workarea);
        return 1, $msg;
    }

    if ($test)
    {
        # Setup the custom add-on on this host and test with run time args.
        ($ret, $msg) = test_custom_addon($addon_dir_fqn, $data);

        if (defined $query->param('remove-test-config') && $query->param('remove-test-config') == 1)
        {
            # delete the installed plugin
            $cmd = VRTS::Util::make_command($perl, "setup.pl", "--operation", "uninstall");
            $out = `$cmd 2>&1`;
            if ($?)
            {
                VRTS::Logger::log('error', "Failed to uninstall the plug after tests: $out");
            }
        }
        # Delete run time configuration parameters from the plugin configuration file
        # and re-create it.
        foreach my $d (keys %{$data->{DEVICE_CONFIG_META}})
        {
            delete $data->{DEVICE_CONFIG_META}->{$d}->{test_value};
        }

        my $config_data;
        eval {
            $config_data = JSON::to_json($data, {pretty => 1, indent => 1});
        };
        if ($@)
        {
            $msg = "Internal Error";
            VRTS::Logger::log('error', "Failed to convert config obj to json: $@\n");
            chdir $tmp_sdkdir;
            File::Path::rmtree($workarea);
            return 1, $msg;
        }
        my $config_file = File::Spec->catfile ($addon_dir_fqn, $config_fname);
        if (open (H, ">$config_file") == 0)
        {
            $msg = "Failed to create plugin config script\n";
            VRTS::Logger::log('error', $msg);
            chdir $tmp_sdkdir;
            File::Path::rmtree($workarea);
            return 1, $msg;
        }
        binmode(H);
        print H $config_data;
        close(H);
    }

    # create the sfa tar
    eval {
        VRTS::SISdk::Fabricate::create_plugin_sfa($workarea, $addon_id);
    };
    if ($@)
    {
        $msg = "Failed to create plugin tar\n";
        VRTS::Logger::log('error', $msg.":$@");
        chdir $tmp_sdkdir;
        File::Path::rmtree($workarea);
        return 1, $msg;
    }

    my $addon_location = File::Spec->catfile ($workarea, $addon_id.".sfa");

    # At this point the workarea has just the add-on sfa file. 
    # The caller should remove the workarea once the Addon file is picked up

    foreach my $file (@discovery_files)
    {
        next if (!-f $file);

        my $basename = File::Basename::basename($file);
        my_print("Check <A href='/logs/$basename' target='_blank'>Enclosure discovery log file</A> for details");
    }
    if (defined $test_log && -f $test_log)
    {
        my $test_filename = File::Basename::basename($test_log);
        my_print("<A href='/logs/$test_filename' target='_blank'>Download test results</A>");
    }
    return 0, $addon_location;
}

# Description: Method tests the SI Plugin with test device configuration parameters.
# It installs the SI Plugin on this box and then probes the array.
# If probe succeeds then it perform deep discovery for each of the enclosures discovered by probe.
# Input :
#       stage_dir - the location where the SI Plugin Add-on is extracted
#       family - the family of the SI Plugin
#       config -  hash which represents the configuration data.
#                 config data is expected to contain the device configuration parameters.
#       TODO: encrypt the password fields in the config file and decrypt them during test execution
#       the method will be later enhanced to not take family and config parameters as input
#       but to read them from the state_dir when the high level test option 
#       is supported in the SDK GUI
# Output :
#     retval : 0 on success. nonzero on failure to invoke test execution
#     msg : Error msg on failure
#     the test execution messages and results are printed on the screen
sub test_custom_addon
{
    my $stage_dir = shift;
    my $config = shift;
    my $msg;

    $ENV{Store} = $stage_dir;
    $ENV{context} = "sdktest";

    chdir $stage_dir;

    my_print ("Installing the SI plugin on this host...");
    my $cmd = VRTS::Util::make_command($perl, "setup.pl");
    my $out = `$cmd 2>&1`;
    if ($?)
    {
        $msg = "Failed to install the plugin : $out";
        VRTS::Logger::log('error', $msg);
        my_print ($msg);
        my_print("Check <A href='/logs/siplugin-setup.log' target='_blank'>Plugin install log</A> for details");
        return 1, $msg;
    }

    my $plugin_install_dir = File::Spec->catdir ($installdir, "etc", "SIPlugin", $family);

    my_print ("Plugin agentlet installed at $plugin_install_dir");

    my_print ("\nProbing the array..\n\n");
    $cmd = "\"$xprtlc\" -l https://localhost/admin/cgi-bin/sfme.pl -d operation=probe_array ";
    my $arg_key;
    foreach my $key (keys %{$config->{DEVICE_CONFIG_META}})
    {
         # ignore parameters than have no test value
         next if (!exists($config->{DEVICE_CONFIG_META}->{$key}->{test_value}));     
         $cmd .= " -d $key=$config->{DEVICE_CONFIG_META}->{$key}->{test_value}";
    }
    $cmd .= " -d control-host=localhost -d method=$family -d context=sdktest -d test-source=$test_source";
    if (defined $test_source_dir)
    {
         my $basename = File::Basename::basename($test_source_dir);
         $cmd .= " -d test-source-dir=$basename";
    }
    VRTS::Logger::log('debug', "Calling probe operation: $cmd");
    $out = `$cmd 2>&1`;
    if ($?)
    {
        $msg = "Failed to invoke the probe operation: $out";
        VRTS::Logger::log('error', "Failed to invoke the probe operation: $out");
        my_print ($msg);
        my_print("Check <A href='/logs/sisdk.log' target='_blank'>SDK log</A> for details");
        return 1, $msg;
    }
    if ($out !~ m/(.+)\n(\{\"RESULT\":.+)\n/)
    {
        $msg = "Unexpected result returned by probe: $out";
        VRTS::Logger::log('error', "Unexpected result returned by probe: $out");
        my_print ($msg);
        my_print("Check <A href='/logs/sisdk.log' target='_blank'>SDK log</A> for details");
        return 1, $msg;
    }
    my $job_hash;
    my $res_hash;
    eval {
        $job_hash = JSON::from_json($1);
    };
    if ($@)
    {
        $msg = "Failed to parse probe output: $@";
        VRTS::Logger::log('error', "Failed to parse probe job output $1.Error $@");
        my_print ($msg);
        my_print("Check <A href='/logs/sisdk.log' target='_blank'>SDK log</A> for details");
        return 1, $msg;
    }
    eval {
        $res_hash = JSON::from_json($2);
    };
    if ($@)
    {
        $msg = "Failed to probe the array";
        VRTS::Logger::log('error', "Failed to parse probe job result $2.Error $@");
        my_print ($msg);
        my_print("Check <A href='/logs/sisdk.log' target='_blank'>SDK log</A> for details");
        return 1, $msg;
    }
    if ($res_hash->{RESULT}->{RETURNCODE} != 0)
    {
        my $inc_agentlet = File::Spec->catfile($installdir, "agentlet");
        push @INC, $inc_agentlet;

        my $probe_err = $res_hash->{RESULT}->{ERROR};
        my $err = $probe_err;
        if (!defined $err)
        {
            $err = $res_hash->{RESULT}->{RETURNCODE};
        }
        if(eval "require VRTS::Enclosure::SIPlugin::Util")
        {
            eval {
                $err = VRTS::Enclosure::SIPlugin::Catalog->$_ for $err;
                VRTS::Logger::log('debug', "error is $err..probe_err is $probe_err");
                if (defined $err)
                {
                    $probe_err = &{\&{"VRTS::Enclosure::SIPlugin::Util::get_error"}}($res_hash->{RESULT}->{ERROR},
                            $err);
                }
            };
        }
        if (defined $err)
        {
            $probe_err = &{\&{"VRTS::Enclosure::SIPlugin::Util::get_error"}}($res_hash->{RESULT}->{ERROR},
                            $err);
        }

        $msg = "Failed to probe the array: $probe_err";
        VRTS::Logger::log('error', $msg);
        my_print ($msg);
        my_print("Check <A href='/logs/mhrun.log' target='_blank'>Probe log</A> for details");
        return 1, $msg;
    } 
    my $enc_hash;
    eval {
        $enc_hash =  JSON::from_json($job_hash->{JOB}->{OUTPUT});
    };
    if ($@)
    {
        $msg = "Failed to probe array";
        VRTS::Logger::log('error', "No Enclosures discovered : $job_hash->{JOB}->{OUTPUT}");
        my_print ($msg);
        my_print("Check <A href='/logs/mhrun.log' target='_blank'>Probe log</A> for details");
        return 1, $msg;
    }

    my @enc_count = keys %$enc_hash;
    if (scalar @enc_count == 0)
    {
        $msg = "No Enclosures were discovered via specified commands";
        VRTS::Logger::log('error', $msg);
        my_print ($msg);
        my_print("Check <A href='/logs/mhrun.log' target='_blank'>Probe log</A> for details");
        return 1, $msg;
    }
    foreach my $enc (keys %$enc_hash)
    {
        my_print("   Probed enclosure : $enc");
    }
    foreach my $enc (keys %$enc_hash)
    {
        my_print ("Performing deep discovery for Enclosure $enc...");
        my $ctxt = "execution-ctxt=sdktest:test-source=$test_source:vardir=$enc";

        if ($test_source == 1)
        {
            my $basename = File::Basename::basename($test_source_dir);
            $ctxt .= ":test-source-dir=$basename";
        }
        if (defined $test_log)
        {
            my $basename = File::Basename::basename($test_log);
            $ctxt .= ":test-log=$basename";
        }
        $cmd = VRTS::Util::make_command($perl, $array_driver, "--hidden", "--family", $family, "--context",
                  "\"$ctxt\"");
        VRTS::Logger::log('debug', "Calling $cmd for deep discovery");
        if (CORE::open (H, "$cmd 2>&1 |") == 0)
        {
            my_print ("Deep discovery failed for enclosure $enc: $!");
            VRTS::Logger::log('error', "Failed to discover enclosure $enc $!");

            my_print("Check <A href='/logs/sisdk.log' target='_blank'>SDK log</A> for details");
        }
        while (<H>)
        {
           my_print ($_, 2);
        }

        my $discovery_file = File::Spec->catdir($app_dir, $family, $enc, "log", "mh.log");
        my $target_file = File::Spec->catfile ($app_dir,"logs", $enc.".log");
        if ( -f $discovery_file)
        {
            File::Copy::copy ($discovery_file, $target_file);
            push (@discovery_files, $target_file);
        }

        close (H);
    }
    return 0, $msg;
}

# Reads the plugin configuration file 
sub read_config
{
    my $config_file = shift;
    my $msg;

    open (H, $config_file) or
      return (1, "failed to open file $config_file : $!");
    my $buf = do { local $/; <H> };
    close (H);

    my $data;
    eval { $data = JSON::from_json($buf)};
    if ($@)
    {
        $msg = "Invalid configuration file\n";
        VRTS::Logger::log('error', $msg.":$@");
        return (1, $msg);
    }
    my %req_input =
    (
        "ARRAY_PLUGIN_NAME" => 1,
        "ARRAY_PLUGIN_VERSION" => 1,
        "ARRAY_PLUGIN_METHOD" => 1,
        "VENDOR" => 1,
        "MODEL" => 1,
        "DISCOVERY_INFO" => ["ENCL_INFO"]
    );

    foreach my $sec (keys %req_input)
    {
        if (!exists $data->{$sec})
        {
            $msg = "Field $sec required in configuration file";
            VRTS::Logger::log('error', $msg);
            return (1, $msg);
        }
    }

    $discv_method = $data->{ARRAY_PLUGIN_METHOD};
    $family =  $discv_method;
    $vendor = $data->{VENDOR};
    $model = $data->{MODEL};

    if (defined $vendor && defined $model)
    {
        $family = ${vendor}."_".${model}."_".${family};
        $family =~ tr/[a-z]/[A-Z]/;
    }
    return (0, $data);
}

# Sets up the stage area for the plugin add-on.
# copies the files to be bunlded in the add-on in the stage area
sub setup_stage_area
{
    my $workarea = shift;
    my $data = shift;
    my $msg;

    my $add_on = "SI_".$family;
    my $soln_ver = $data->{'ARRAY_PLUGIN_VERSION'};
    $soln_ver =~ s/'//g;
    $soln_ver =~ s/^v//;

    $addon_id = $add_on."-".$soln_ver;
    my $addon_dir_fqn = File::Spec->catdir($workarea, $addon_id);
    my $sisdk_dir = File::Spec->catfile($installdir, "etc", "SISdk");
    eval ( File::Path::mkpath($addon_dir_fqn, 0, 0755) );
    if($@)
    {
        $msg = "Internal Error";
        VRTS::Logger::log('debug',"Failed to create directory $addon_dir_fqn:$@");
        return 1, $msg;
    }
    my @plugin_addon_files = (
        "setup.pl", "unsetup.pl", "tdf" , "SIPlugin", "SIBase", "cliobjects.def",
        "enable.pl", "disable.pl"
    );
    my $catalog_file = File::Spec->catfile ($addon_dir_fqn, "catalog.ini");
    if (open (H, ">$catalog_file") == 0)
    {
        $msg = "Failed to create catalog.ini $catalog_file\n";
        VRTS::Logger::log('debug', $msg);
        return 1, $msg;
    }
    binmode (H);
    print H "[ADDON]\n".
             "NAME=\"$addon_id\";\n".
             "VERSION=\"$soln_ver\";\n".
             "FRIENDLY_NAME=\"$data->{ARRAY_PLUGIN_NAME}\";\n".
             "DESCRIPTION = \"SI Plugin for Vendor $vendor Product $model using $discv_method ".
                             "as the discovery method\";\n".
             "TYPE = Addon;\n".
             "MH_COMPONENT = N;\n".
             "CS_COMPONENT  = Y;\n".
             "CUMULATIVE = 1;\n".
             "ENABLE_ON_FAILOVER = 1;\n".
             "[UNIX_SCRIPTS]\n".
             "SETUP = setup.pl;\n".
             "UNSETUP = unsetup.pl;\n".
             "ENABLE = enable.pl;\n".
             "DISABLE = disable.pl;\n".
             "[WINDOWS_SCRIPTS]\n".
             "SETUP = setup.pl;\n".
             "ENABLE = enable.pl;\n".
             "DISABLE = disable.pl;\n".
             "UNSETUP = unsetup.pl;\n";
    close (H);

    # copy all the plugin add-on files into the workarea
    foreach my $plugin_file (@plugin_addon_files)
    {
        my $file = File::Spec->catfile ($sisdk_dir, $plugin_file);
        my $dest_file = File::Spec->catfile ($addon_dir_fqn, $plugin_file);
        if (-f $file)
        {
            File::Copy::copy ($file, $dest_file);
        }
        elsif (-d $file)
        {
            eval { VRTS::Util::copy_recursively($file, $dest_file);};
            if ($@)
            {
                $msg = "Internal Error";
                VRTS::Logger::log('error', "Failed to copy $file to $dest_file: $@\n");
                return 1, $msg;
            }
        }
    }
    if (exists $data->{DISCOVERY_INFO}->{COMMAND_LOCATION} &&
               $data->{DISCOVERY_INFO}->{COMMAND_LOCATION} ne "")
    {
        my $dest = File::Spec->catfile ($addon_dir_fqn, $family);
        my $readme =  File::Spec->catfile ($addon_dir_fqn, "Readme.html");

        if (-d $data->{DISCOVERY_INFO}->{COMMAND_LOCATION})
        {
            # if the discovery scripts have to be bundled in the SIPlugin
            # then bundle them in a tar, give executable permissions to them and
            # create a setup script that will install these scripts on the target machine
 
            VRTS::Util::copy_recursively($data->{DISCOVERY_INFO}->{COMMAND_LOCATION}, $dest);
            chdir $dest;
            # tar the discovery scripts.
            my $tar = Archive::Tar->new;
            my @tar_files;
            opendir my($dh), $dest;
            foreach my $entry (readdir $dh)
            {
                next if ($entry eq "." or $entry eq "..");
                if ($entry eq "Readme.html")
                {
                    my $readme_src = File::Spec->catfile ($data->{DISCOVERY_INFO}->{COMMAND_LOCATION}, "Readme.html");
                    VRTS::Logger::log('debug' , "Bundling Readme $readme_src \n");
                    File::Copy::copy($readme_src, $readme);
                }
                push @tar_files, $entry;
            }
            closedir $dh;
            $tar->add_files( @tar_files);
            foreach my $f ( $tar->get_files )
            {
                # give executable permissions
                $f->mode( 0755 );
            }
            if (! defined $tar->write( "discovery-scripts.tar.gz", 9))
            {
                $msg = "Internal Error";
                VRTS::Logger::log('error', "Failed to tar the discovery scripts ".$tar->error());
                return 1, $msg;
            }        
            foreach my $file (@tar_files)
            {
                unlink($file);
            }
            my $setup_script = File::Spec->catfile ($dest, "setup-modules.pl");
            open F, ">$setup_script";
            binmode(F);
            while (<DISCOVERY_MODULE_SETUP>)
            {
                print F $_;
            }
            close (F);
         }
    }

    # create the plugin config file
    my $config_file = File::Spec->catfile ($addon_dir_fqn, $config_fname);

    $data->{SDK_VERSION} = $sdk_version;

    # Update the config file with the family that will be used for this plugin
    $data->{ARRAY_PLUGIN_FAMILY} = $family;
    $data->{PLUGIN_ADDON_ID} = $addon_id;

    if (defined $data->{ID_DEFS})
    {
        # Update the identifier definitions with the page_list key that 
        # specifies the list of pages required for the given scsi vendorid/scsi product id
        # combination
        my @vendorids = keys %{$data->{ID_DEFS}};
        if (scalar(@vendorids) > 0) 
        {
            foreach my $vendorid (@vendorids)
            {
                foreach my $productid (keys %{$data->{ID_DEFS}->{$vendorid}})
                {
                    my @page_list;
                    foreach my $field (keys %{$data->{ID_DEFS}->{$vendorid}->{$productid}->{FIELDS}})
                    {
                        push (@page_list, 
                          $data->{ID_DEFS}->{$vendorid}->{$productid}->{FIELDS}->{$field}->{pgno});
                    }
                    if (scalar(@page_list) > 0)
                    {
                        $data->{ID_DEFS}->{$vendorid}->{$productid}->{PAGE_LIST} = \@page_list;
                    }
                }
            }
        }
    } 
    my $config_data;
    eval { $config_data = JSON::to_json($data, {pretty => 1, indent => 1}); };
    if ($@)
    {
        $msg = "Internal Error";
        VRTS::Logger::log('error', "Failed to convert config obj to json: $@\n");
        return 1, $msg;
    }
    if (open (H, ">$config_file") == 0)
    {
        $msg = "Failed to create plugin config script\n";
        VRTS::Logger::log('error', $msg);
        return 1, $msg;
    }
    binmode (H);
    print H $config_data;
    close(H);
    return 0, $addon_dir_fqn;
}

sub my_print
{
    my $msg = shift;
    my $format = shift;

    $format = 1 if (!defined $format);
    if ($format == 1)
    {
        $msg = "<div> <div style='color: green;'><br><b>$msg</b></div></div>\n";
    }
    $fh->print($msg);
    $fh->flush;
}


__DISCOVERY_MODULE_SETUP__
#!/opt/VRTSsfmh/bin/perl
BEGIN { @INC = ("/opt/VRTSsfmh/lib/modules") if ($^O !~ /Win32/i);}
require VRTS::OS;
require File::Basename;
require File::Path;
require File::Spec;
require VRTS::Paths;
require File::Copy;

my $tar_location = "##TAR_LOCATION##";
my $check_env_args = "##ARGS##";
my $family = "##FAMILY##";

my $install_dir = VRTS::Paths::get_path("InstallDir");

my $plugin_install_dir = File::Spec->catdir ($install_dir, "etc", "SIPlugin", $family);
my $tar_name = File::Basename::basename ($tar_location);
my $dest_file = File::Spec->catfile ($plugin_install_dir, $tar_name);
my $tar_file = $dest_file =~ s/\.tar\.gz$/\.tar/;
my $ret = 0;
my $msg;

if ( ! -d $plugin_install_dir)
{
    eval ( File::Path::mkpath($plugin_install_dir, 0, 0755) );
    if($@)
    {
        $msg = "Failed to create dir $plugin_install_dir:$@";
        $ret = 2;
        goto out;
     }
}

File::Copy::copy($tar_location, $dest_file);
if (VRTS::OS->ungzip_and_untar($plugin_install_dir, $dest_file))
{
    $msg = "Failed to ungzip tar modules\n";
    $ret = 2;
    goto out;
}
my $perl = File::Spec->catfile ($install_dir, "bin", "perl");
my $check_env_cmd = File::Spec->catfile($plugin_install_dir, "check_env.pl");
if (-f $check_env_cmd)
{
    $ENV{SI_PLUGIN_DIR} = $plugin_install_dir;
    my $cmd = "\"$perl\" \"$check_env_cmd\" $check_env_args";
    my $data = `$cmd 2>&1`;
    if ($?)
    {
        $msg = "Plugin checkenv command failed with error: $data\n";
        $ret = 3;
        goto out;
    }
}

out:
    unlink ($tar_location)  if (-f $tar_location);
    unlink ($dest_file) if (-f $dest_file);
    unlink ($tar_file) if (-f $tar_file);
    if ($ret)
    {
        print STDERR "Failed to setup discovery scripts:\n $msg";
    }
    exit ($ret);
